'##################################################################################################
'Code Settings
SuperStrict

'##################################################################################################
'Framework & Imports
Framework Leadwerks.ENGINE
Import Pub.FreeJoy
Import BRL.Timer
Import BRL.Map

'##################################################################################################
'Includes
Include "..\Source\GlobalVariables.bmx"
Include "..\Source\Classes\TApp.bmx"
Include "..\Source\Classes\TAnimation.bmx"
Include "..\Source\Classes\TCharacter.bmx"
Include "..\Source\Classes\TController.bmx"
Include "..\Source\Classes\TRenderer.bmx"

'##################################################################################################
'Init
TApp.Initialize()
TRenderer.Initialize()
TApp.PaintSkyBox( LoadMaterial( TApp.AbstractZipFile( "Textures\Skybox.mat" ) ) )

Global MyChar:TCharacter = TCharacter.Create( TApp.AbstractZipDir( "Models\Characters\MarioGalaxy\" ) )
MyChar.EnableEyeLid( 4 )

Function TempLoadAnimations:Int( FilePath:String )
	If FileType( FilePath ) <> FILETYPE_FILE Then Notify( "File '" + FilePath + "' doesn't exist." ); Return False
	
	Local State:Int = 0
	Local File:TStream = ReadFile( FilePath )
		While Not Eof( File )
			Local Line:String = ReadLine( File )
			Select State
				Case 0
					Select Upper( Line )
						Case "<<ANIM_START>>"
							Local AnimSF:Int = Int( ReadLine( File ) )
							Local AnimEF:Int = Int( ReadLine( File ) )
							MyChar.AddAnim( "", AnimSF, AnimEF )
							State = 1
					End Select
				Case 1
					Select Upper( Line )
						Case "<<ANIM_END>>"
							State = 0
					End Select
			End Select
		Wend
	CloseFile( File )
End Function

TempLoadAnimations( "Animations.txt" )

Global TestLevel:TMesh = LoadMesh( TApp.AbstractZipFile( "Levels\TempLevel\mesh.gmf" ) )

Global CamPivot:TPivot = CreatePivot()
EntityParent( TApp.MainCam, CamPivot )
Global CamOffset:Float = 3
TurnEntity( CamPivot, Vec3( 0, 180, 0 ) )
Global TempPivot:TPivot = CreatePivot()
PositionEntity( TempPivot, Vec3( 0, 0.5, 0 ) )

Global Light:TDirectionalLight = CreateDirectionalLight()
PositionEntity( Light, Vec3( 0, 100, 100 ) )
RotateEntity( Light, Vec3( 45, 225, 45 ) )
SetShadowmapSize( Light, 4096 )
SetShadowSoftness( Light, 4 )

Global DebugAnimFrame:Float = 0
Global DebugAnimSpeed:Float = 0.5
Global DebugAnimOn:Int = 1
Global DebugAnimNr:Int = 0
Global DebugAnim:TAnimation
If MyChar.AnimCount > 0
	DebugAnim = MyChar.AnimationFromID( 0 )
EndIf

'##################################################################################################
'Main Loop
While Not KeyHit( KEY_ESCAPE )
	
	MyChar.Update()
	
	If KeyHit( KEY_ENTER )
		If DebugAnimOn = 0
			DebugAnimOn = 1
		ElseIf DebugAnimOn = 1
			DebugAnimOn = 0
		EndIf
	EndIf
	
	Select DebugAnimOn
		Case 0
			
			If KeyHit( KEY_UP )
				If DebugAnimNr + 1 < MyChar.AnimCount
					DebugAnimNr:+1
					DebugAnim = MyChar.AnimationFromID( DebugAnimNR )
				EndIf
			ElseIf KeyHit( KEY_DOWN )
				If DebugAnimNr - 1 > 0
					DebugAnimNr:-1
					DebugAnim = MyChar.AnimationFromID( DebugAnimNR )
				Else
					DebugAnimNr = 0
					DebugAnim = MyChar.AnimationFromID( DebugAnimNR )
				EndIf
			EndIf
			
			If DebugAnim <> Null
				MyChar.Animate( DebugAnim )
			EndIf
			
		Case 1
			If KeyHit( KEY_NUM8 )
				DebugAnimFrame = Int( DebugAnimFrame ) + 1
			ElseIf KeyHit( KEY_NUM5 )
				If Int( DebugAnimFrame ) - 1 > 0
					DebugAnimFrame = Int( DebugAnimFrame ) - 1'
				Else
					DebugAnimFrame = 0.0
				EndIf
			EndIf
			
			If KeyDown( KEY_UP )
				DebugAnimFrame:+ ( DebugAnimSpeed * AppSpeed() )
			ElseIf KeyDown( KEY_DOWN )
				If DebugAnimFrame - ( DebugAnimSpeed * AppSpeed() ) > 0
					DebugAnimFrame:- ( DebugAnimSpeed * AppSpeed() )
				Else
					DebugAnimFrame = 0.0
				EndIf
			EndIf
			
			If KeyHit( KEY_NUMADD )
				DebugAnimSpeed:+0.1
			ElseIf KeyHit( KEY_NUMSUBTRACT )
				If DebugAnimSpeed - 0.1 > 0
					DebugAnimSpeed:-0.1
				Else
					DebugAnimSpeed = 0
				EndIf
			EndIf
			
			TAnimation.Debug( MyChar.Mesh, DebugAnimFrame )
	End Select
	
	If KeyDown( KEY_S )
		If CamOffset < 10
			CamOffset = CamOffset + 0.1 * AppSpeed()
		EndIf
	ElseIf KeyDown( KEY_W )
		If CamOffset > 1
			CamOffSet = CamOffset - 0.1 * AppSpeed()
		EndIf
	EndIf
	
	If KeyDown( KEY_A )
		TurnEntity( CamPivot, Vec3( 0, -1 * AppSpeed(), 0 ) )
	ElseIf KeyDown( KEY_D )
		TurnEntity( CamPivot, Vec3( 0, 1 * AppSpeed(), 0 ) )
	EndIf
	
	PositionEntity( TApp.MainCam, Vec3( 0, 1 + CamOffset/2, CamOffset ) )
	PointEntity( TApp.MainCam, TempPivot )
	
	UpdateWorld()
	TRenderer.Render()
	
	UpdateAppTime()
	
Wend

'##################################################################################################