#extension GL_ARB_draw_buffers : enable

uniform vec3 cameraposition;
uniform vec2 buffersize;
uniform vec2 camerarange;

include "abstract::greyscale.txt"
include "abstract::DepthToZPosition.frag"

#ifdef LW_FALLOFF
	uniform float falloffFactor = 0.05;
	uniform float falloffIntensity = 10.0;
	uniform vec4 falloffColor = vec4(1.0, 1.0, 0.9, 1.0);
	uniform float falloffEdgeTurnOff = 0.1;
	varying vec3 modelnormal;
#endif

#ifdef LW_DETAIL
	uniform sampler2D LW_DETAIL;
#endif

#ifdef LW_DIFFUSE
	uniform sampler2D LW_DIFFUSE;
#endif

#ifdef LW_SPECULARMAP
	uniform sampler2D LW_SPECULARMAP;
#endif

#ifdef LW_BUMPMAP
	uniform sampler2D LW_BUMPMAP;
#endif

#ifdef LW_BLOOM
	uniform sampler2D LW_BLOOM;
#endif

#ifdef LW_CUBEMAP
	uniform samplerCube LW_CUBEMAP;
#endif

#ifdef LW_PARALLAXMAP
	uniform sampler2D LW_PARALLAXMAP;
	varying vec3 eyevec;
#endif

#ifdef LE_REFRACTION
	uniform sampler2D LE_REFRACTION;
	uniform sampler2D LE_DEPTHBUFFER;
	uniform float refractionstrength = 0.01;
#endif

#ifdef LW_POMMAP
	vec3 vLightTS=vec3(0.577,0.577,0.577);
	varying vec3 eyevec;
	float depthP = .15;
	float nMinSamples = 20;
	float nMaxSamples = 50;	
#endif

#ifdef LW_MESHLAYER
	varying float vegetationfade;
#endif

#ifdef LW_ALPHABLEND
	uniform sampler2D LW_ALPHABLEND_INCOMINGCOLOR;
	uniform sampler2D LW_ALPHABLEND_INCOMINGNORMAL;
#endif

varying vec3 vertexposition;
varying vec3 T,B,N;
varying vec2 texcoord0;
varying vec2 texcoord1;
varying vec3 cubecoord;
varying vec4 modelvertex;
varying vec4 fragcolor;

float fOcclusionShadow = 1.0;

uniform sampler2D texture14;
uniform float terrainsize;
uniform vec3 terrainscale;
uniform float bumpscale;
uniform float specular;
uniform float gloss;

//Terrain color map
uniform sampler2D texture12;

void main(void) {
	
	vec4 diffuse = fragcolor;
	vec3 normal;
	float shininess = 0.0;
	vec2 texcoord=texcoord0;// only use this because of parallax mapping
	float selfillumination = 0.0;	

	vec2 terraincoord;
	float terrainresolution;
	
	#ifdef LW_PARALLAXMAP
		texcoord += (texture2D(LW_PARALLAXMAP,texcoord).x * 0.04 - 0.036) * normalize(eyevec).xy;
	#endif
	
	#ifdef LW_POMMAP
		// for POM, the heightmap is in the alpha of the diffuse so save ur diffuse with DXT5 I chose this because the alpha of DXT5 is higher precision
		Include "abstract::POM.frag"
	#endif

	#ifdef LW_DIFFUSE
		diffuse *= texture2D(LW_DIFFUSE,texcoord);//*fOcclusionShadow;
	#endif

	#ifdef LW_ALPHATEST
		if (diffuse.w<0.5) {
			discard;
		}
	#endif

	#ifdef LW_FALLOFF
		// apply falloff effect
		// calculate normal and normal to camera
		vec3 eyeDir = normalize(cameraposition - vertexposition.xyz);   
		vec3 nmodelnormal = normalize(modelnormal);
   
		// use dot product to calculate falloff
		float dp = max(dot(eyeDir,nmodelnormal),falloffEdgeTurnOff);
		float falloffValue = 1.0 - clamp(pow(dp,falloffFactor), 0.0, 1.0);
   
		diffuse = mix(diffuse,falloffColor,falloffValue*falloffIntensity);
	#endif

	normal = N;
	
	#ifdef LW_BUMPMAP

		#ifdef LW_TERRAINNORMALS
			//Use terrain normals
			terraincoord=vec2(vertexposition.x,-vertexposition.z) / terrainsize + 0.5;
			terrainresolution = terrainsize / terrainscale.x;
			terraincoord += 0.5 / terrainresolution;
			vec3 worldNormal = ((texture2D(texture14,terraincoord).xyz - 0.5) * 2.0).xyz;
			normal = normalize(gl_NormalMatrix*worldNormal);
		#else
			vec4 bumpcolor = texture2D(LW_BUMPMAP,texcoord);
			normal = bumpcolor.xyz * 2.0 - 1.0;
		#endif
		
		#ifdef LW_DETAIL
			normal += texture2D(LW_DETAIL,texcoord * 4.0).xyz * 2.0 - 1.0;
		#endif
		normal.z /= bumpscale;
		normal = T * normal.x + B * normal.y + N * normal.z;
		normal = normalize(normal);
		#ifdef LW_SPECULAR
			shininess = bumpcolor.a*specular;//*fOcclusionShadow
		#endif
		#ifdef LW_SPECULARMAP
			shininess = texture2D(LW_SPECULARMAP,texcoord).x*specular;//*fOcclusionShadow
		#endif
	
	#else
		normal=normalize(normal);
	#endif

	#ifdef LW_TERRAINNORMALS
		#ifndef LW_BUMPMAP
			//Use terrain normals
			terraincoord=vec2(vertexposition.x,-vertexposition.z) / terrainsize + 0.5;
			terrainresolution = terrainsize / terrainscale.x;
			terraincoord.x -= 0.5 / terrainresolution;
			//vec4 normsample=((texture2D(texture14,terraincoord).xyz - 0.5) * 2.0).xyz;
			vec3 worldNormal = ((texture2D(texture14,terraincoord).xyz - 0.5) * 2.0).xyz;
			normal = normalize(gl_NormalMatrix*worldNormal);
			//shininess = normsample.w;
		#endif
	#endif

	#ifdef LW_TERRAINCOLOR
		//Use terrain color
		terraincoord=vec2(vertexposition.x,-vertexposition.z) / terrainsize + 0.5;
		terrainresolution = terrainsize / terrainscale.x;
		terraincoord.x -= 0.5 / terrainresolution;
		//terraincoord.y += 0.5 / terrainresolution;
		vec4 terraincolor = texture2D(texture12,terraincoord);
		diffuse = vec4( greyscale(diffuse.xyz) * 2.0 * terraincolor.xyz,diffuse.w);

		#ifdef LW_MESHLAYER
			float temp_w=diffuse.w;
			diffuse = diffuse * (1.0-vegetationfade) + terraincolor * vegetationfade;
			diffuse.w=temp_w;
		#endif

		shininess = terraincolor.w;
	#endif
	
	#ifdef LE_REFRACTION
		diffuse.a=0.25;
		vec4 refractionvector = vec4( gl_FragCoord.x/buffersize.x, gl_FragCoord.y/buffersize.y, gl_FragCoord.z, 1.0 );
		vec4 refractionvector2 = refractionvector + refractionstrength * vec4(normal,0.0);		
		if (gl_FragCoord.z<DepthToZPosition(texture2DProj(LE_DEPTHBUFFER,refractionvector2).x)) {
			refractionvector=refractionvector2;
		}
		vec4 transparency = texture2DProj(LE_REFRACTION,refractionvector);
		diffuse = transparency * diffuse;
	#endif
	
	vec3 adjustednormal = normal*0.5+0.5;
	float adjustedgloss = gloss;

	//Diffuse
	gl_FragData[0] = diffuse;	

	//Normal
	#ifdef LW_FULLBRIGHT
		gl_FragData[1] = vec4(1.0,1.0,1.0,diffuse.w);
	#else
		gl_FragData[1] = vec4(adjustednormal,diffuse.w);
	#endif
	
	//Bloom
	#ifdef LW_BLOOM
		gl_FragData[3] = texture2D(LW_BLOOM,texcoord) * fragcolor;
	#else
		gl_FragData[3] = vec4(0,0,0,diffuse.w);
	#endif
	
	shininess=clamp(shininess,0,1)*0.5;
	
	gl_FragData[2]=vec4(shininess,gloss,0.0,diffuse.w);




	/*
	float c;
	float temp;
	temp=adjustednormal.x*100.0;
	c=int(temp);
	temp=adjustednormal.y*100.0;
	c+=int(temp)/100.0;
	temp=adjustednormal.z*100.0;
	c+=int(temp)/100.0/100.0;
	gl_FragData[1].x = c;
	*/
}
