uniform sampler2D texture0;
uniform vec2 toolposition;
uniform vec2 toolradius;
uniform float toolstrength;
uniform vec2 buffersize;
uniform float flattenheight;
uniform vec2 heightmapsize;
uniform vec2 imagesize;
uniform vec4 channelmask;
uniform vec2 constraints;

varying vec2 texcoord;

void main(void) {
	vec4 height;
	float effect;
	vec2 ps = 1.0/imagesize;
	float d;

	height=texture2D(texture0,texcoord);

	if (height.x<constraints.x) {
		discard;
	}
	
	if (height.x>constraints.y) {
		discard;
	}
	
	effect=length(gl_FragCoord.xy/buffersize-toolposition);

	if (effect>toolradius.x) {
		discard;
	}

	effect = (effect-toolradius.y);

	if (toolradius.x-toolradius.y>0.0) {
		effect /= (toolradius.x-toolradius.y);
	}
	effect = clamp(effect,0.0,1.0);
	effect = cos(effect*1.5707963);
	effect *= toolstrength;
	
	//Sculpt
	#if TOOLMODE==0
		height += 0.001 * effect;
	#endif
	
	//Paint
	#if TOOLMODE==1
		height += 1.0 * effect * channelmask;
	#endif

	//Flatten
	#if TOOLMODE==2
		effect *= 0.1;
		height=height*(1.0-effect)+vec4(flattenheight)*effect;
	#endif

	//Smooth
	#if TOOLMODE==3
		vec4 sheight;
		sheight =height;
		sheight+=texture2D(texture0,texcoord+vec2(-ps.x,-ps.y));
		sheight+=texture2D(texture0,texcoord+vec2(+ps.x,-ps.y));
		sheight+=texture2D(texture0,texcoord+vec2(-ps.x,+ps.y));
		sheight+=texture2D(texture0,texcoord+vec2(+ps.x,+ps.y));
		sheight+=texture2D(texture0,texcoord+vec2(0.0,+ps.y));
		sheight+=texture2D(texture0,texcoord+vec2(0.0,-ps.y));
		sheight+=texture2D(texture0,texcoord+vec2(-ps.x,0.0));
		sheight+=texture2D(texture0,texcoord+vec2(+ps.x,0.0));
		sheight/=9.0;
		effect = clamp(effect,0.0,1.0);
		height = height*(1.0-effect)+sheight*effect;
	#endif
	
	//Custom
	#if TOOLMODE==6
	
		//Erosion
		float aheight;
		float dh;
		aheight =texture2D(texture0,texcoord+vec2(-ps.x,-ps.y)).x;
		dh+=aheight-height.x;
		aheight=texture2D(texture0,texcoord+vec2(+ps.x,-ps.y)).x;
		dh+=aheight-height.x;
		aheight=texture2D(texture0,texcoord+vec2(-ps.x,+ps.y)).x;
		dh+=aheight-height.x;
		aheight=texture2D(texture0,texcoord+vec2(+ps.x,+ps.y)).x;
		dh+=aheight-height.x;
		aheight=texture2D(texture0,texcoord+vec2(0.0,+ps.y)).x;
		dh+=aheight-height.x;
		aheight=texture2D(texture0,texcoord+vec2(0.0,-ps.y)).x;
		dh+=aheight-height.x;
		aheight=texture2D(texture0,texcoord+vec2(-ps.x,0.0)).x;
		dh+=aheight-height.x;
		aheight=texture2D(texture0,texcoord+vec2(+ps.x,0.0)).x;
		dh+=aheight-height.x;
		height += (dh*0.5 / 8.0);
		//height = height*(1.0-effect) + effect * (dh*0.5 / 8.0);
		
	#endif
	
	height = clamp(height,constraints.x,constraints.y);
	
	gl_FragColor = height;

	//gl_FragColor = vec4(0.5);

}