#extension GL_ARB_draw_buffers : enable

//==========================================
// Directional Light
//==========================================

uniform sampler2D texture0;//color
uniform sampler2D texture1;//depth
uniform sampler2D texture2;//normal
uniform sampler2DShadow texture3;//shadowmap
uniform sampler2D texture4;//cloud texture
uniform sampler2D texture5;//noise
uniform vec2 shadowmapsize;

uniform vec4 mappingaxis[2];

uniform vec2 buffersize;
uniform vec2 camerarange;
uniform float camerazoom;
uniform mat3 lightmatrix0;
uniform mat3 lightmatrix1;
uniform mat3 lightmatrix2;
uniform mat3 lightmatrix3;
uniform vec3 lightposition0;
uniform vec3 lightposition1;
uniform vec3 lightposition2;
uniform vec3 lightposition3;
uniform vec3 lightdirection;
//uniform float shadowsoftness;
#define shadowsoftness 1.0

uniform float rnd;

uniform vec4 shadowrange;

uniform float apptime;

uniform float shadowjitter;
//#define ShadowJitter 0.001

//varying vec2 texcoord;

Include "DepthToZPosition.frag"
Include "Shadow2D4.frag"

void main(void) {

	float shadowjitter_=shadowjitter;

	vec2 texcoord = gl_FragCoord.xy/buffersize;

	vec4 diffuse = texture2D(texture0,texcoord);
	float depth = texture2D(texture1,texcoord).x;
	vec4 cnormal = texture2D(texture2,texcoord);
	vec3 normal = cnormal.xyz * 2.0 - 1.0;
	
	//normal = vec3(0.0,0.0,-1.0);
	
	//normal = normalize( normal );

	vec4 ambientlight = gl_LightSource[0].ambient;
	vec4 lightcolor = gl_LightSource[0].diffuse;
	float attenuation = 1.0 - int(depth); // simple way of saying if depth=1.0 don't do any lighting (background or empty)
	ambientlight = ambientlight * 1.0-int(depth) + int(depth);
	attenuation *= clamp( dot( normal, lightdirection ),0.0,1.0);
	lightcolor *= attenuation;
	
	float specular = 0.0;
	float shadowcolor = 1.0;
	
	if ((cnormal.x+cnormal.y+cnormal.z)==3.0) {
		attenuation=0.0;
		depth=1.0;
	}

	vec3 screencoord;
	screencoord = vec3(((gl_FragCoord.x/buffersize.x)-0.5) * 2.0,((-gl_FragCoord.y/buffersize.y)+0.5) * 2.0 / (buffersize.x/buffersize.y),DepthToZPosition( depth ));
	//screencoord = vec3(((texcoord.x)-0.5) * 2.0,((-texcoord.y)+0.5) * 2.0 / (buffersize.x/buffersize.y),DepthToZPosition( depth ));
	
	screencoord.x *= screencoord.z / camerazoom;
	screencoord.y *= -screencoord.z / camerazoom;
	
	#ifdef __GLSL_CG_DATA_TYPES
	if (attenuation>0.0) {
	#endif
				
		vec3 smoffset;
		mat3 lightmatrix;
		vec3 lightposition;
		
		shadowjitter_ /= 4.0;

		float l=length(screencoord);
		if (l<shadowrange.z) {
			lightmatrix = lightmatrix2;
			smoffset=vec3(0.0,0.5,0.0);
			lightposition = lightposition2;
			if (l<shadowrange.y) {
				lightmatrix = lightmatrix1;	
				smoffset=vec3(0.5,0.0,0.0);
				lightposition = lightposition1;
				shadowjitter_ *= 2.0;
			}
			if (l<shadowrange.x) {
				lightmatrix = lightmatrix0;
				smoffset=vec3(0.0,0.0,0.0);
				lightposition = lightposition0;
				shadowjitter_ *= 2.0;
			}
			
			vec3 smcoord;
			
			float zFar = gl_LightSource[0].linearAttenuation;
			float zNear = 0.1;
			
			float shadowMapResolution = gl_LightSource[0].quadraticAttenuation;
			float psx = shadowsoftness / (shadowMapResolution * 2.0);
			float psy = shadowsoftness / (shadowMapResolution * 2.0);
			
			smcoord = lightmatrix * ( lightposition - screencoord );
			smcoord.x /= -1.0;
			smcoord.y /= -1.0;
			smcoord.x += 0.5;
			smcoord.y += 0.5;
			smcoord.z = -(smcoord.z-zNear) / (zFar-zNear);
			smcoord.x *= 0.5;
			smcoord.y *= 0.5;
			smcoord += smoffset;
			
			Include "ShadowLookup.frag"
			
			shadowcolor += max( (l - (shadowrange.z-10.0) ) / 10.0, 0.0 );
			shadowcolor = min(shadowcolor,1.0);
						
		}
		
		#ifdef LW_SPECULAR
		vec3 R = -reflect(vec3(lightdirection.x,lightdirection.y,-lightdirection.z), vec3(normal.x,normal.y,-normal.z));
		specular = pow(clamp(dot(R, normalize(-screencoord) ),0.0,1.0),8.0) * attenuation * gl_LightSource[0].diffuse.w * cnormal.w;

		//vec3 halfvec = normalize( vec3(lightdirection.x,lightdirection.y,-lightdirection.z) - normalize(screencoord) );
		//specular = pow(max(0.0, dot(halfvec,vec3(normal.x,normal.y,-normal.z))),24.0) * cnormal.w * gl_LightSource[0].diffuse.w;
		#endif
		
	#ifdef __GLSL_CG_DATA_TYPES
	}
	#endif
	
	#ifdef LW_LIGHTTEXTURE
	lightcolor *= texture2D(texture4,vec2( dot(mappingaxis[0].xyz,screencoord) + mappingaxis[0].w, dot(mappingaxis[1].xyz,screencoord) + mappingaxis[1].w ));
	#endif
	
	gl_FragData[0] = diffuse * (lightcolor * shadowcolor + ambientlight) + specular * shadowcolor;
	//gl_FragData[1] = vec4(specular*diffuse);
	
	//else {
	//	gl_FragColor = ambientlight * diffuse;
	//}

	if (depth==1.0) {
		#ifdef LW_WRITEDEPTH
			gl_FragData[0] = diffuse;
		#else
			discard;
		#endif
	}

	#ifdef LW_WRITEDEPTH
	//gl_FragDepth = depth;
	#endif
	
}


