#extension GL_ARB_draw_buffers : enable
#define shadowsoftness 1.0

//==========================================
// Directional Light
//==========================================

varying vec4 fragcoord_;

uniform sampler2D texture0;//color
uniform sampler2D texture1;//depth
uniform sampler2D texture2;//normal
uniform sampler2D texture3;//specular/gloss
uniform sampler2DShadow texture4;//shadowmap
uniform sampler2D texture5;//noise
uniform sampler2D texture6;//cloud texture
uniform sampler2D texture7;// glow map
uniform sampler2DShadow texture8;// secondary depth buffer for colored shadows

uniform vec2 shadowmapsize;
uniform vec4 mappingaxis[2];
uniform vec2 buffersize;
uniform vec2 camerarange;
uniform float camerazoom;
uniform mat3 lightmatrix0;
uniform mat3 lightmatrix1;
uniform mat3 lightmatrix2;
uniform mat3 lightmatrix3;
uniform vec3 lightposition0;
uniform vec3 lightposition1;
uniform vec3 lightposition2;
uniform vec3 lightposition3;
uniform vec3 lightdirection;
uniform float rnd;
uniform vec4 shadowrange;
uniform float apptime;
uniform float shadowjitter;

Include "DepthToZPosition.frag"
Include "Shadow2D4.frag"

void main(void) {
	
	vec2 noise;
	float sw;
	float sh;	
	
	float fbool;
	float shadowjitter_=shadowjitter;
	vec2 texcoord = gl_FragCoord.xy/buffersize;
	vec4 diffuse = texture2D(texture0,texcoord);
	float depth = texture2D(texture1,texcoord).x;
	vec4 cnormal = texture2D(texture2,texcoord);
/*
float temp;
float c;
c=cnormal.x;
temp=int(c);
cnormal.x=temp/100.0;
c=(c-temp)*100.0;


temp=int(c);
cnormal.y=temp/100.0;
c=(c-temp)*100.0;

cnormal.z=c/100.0;
*/
float linearshadowdepth;

	#ifdef LW_LIGHTTEXTURE
		vec4 lighttint;
		vec4 atihacklighttint;
	#endif

	#ifdef LW_LIGHTTEXTURE
		vec4 coloredshadowcolor = vec4(1);
	#endif
	
	vec4 sample3 = texture2D(texture3,texcoord);
	float materialspecular = sample3.x;
	float gloss = sample3.y * 16.0;	
	vec4 glowcolor = vec4(0);


	//float selfillumination = sample3.z;

	//float materialspecular = cnormal.w;

	vec3 normal;
	normal = cnormal.xyz * 2.0 - 1.0;

	//Attempt at compressing normal
	//normal.z = sqrt( 1.0 - normal.x*normal.z - normal.y*normal.y) * sign(normal.z);	

	normal = normalize(normal);
	
	//vec3 normal = vec3((cnormal.x-0.5)*2.0,(cnormal.y-0.5)*2.0,0);
	
	

	//float l = (normal.x*normal.x+normal.y*normal.y);
	//normal.z = (cnormal.w-0.5)*2.0;
	//normal.z = sqrt(1.0-l)/2.0;// * float(sign(cnormal.w-0.5));
	//normal.z=sqrt(1.0-(normal.x*normal.x+normal.y*normal.y)) * float(sign(cnormal.w-0.5));

	
	//float gloss = diffuse.w * 16.0;

	//normal = normalize(normal);

	vec4 ambientlight = gl_LightSource[0].ambient;
	vec4 lightcolor = gl_LightSource[0].diffuse;
	float attenuation = 1.0 - int(depth); // simple way of saying if depth=1.0 don't do any lighting (background or empty)
	ambientlight = ambientlight * 1.0-int(depth) + int(depth);
	attenuation *= clamp( dot( normal, lightdirection ),0.0,1.0);
	lightcolor *= attenuation;

	#ifdef LW_GLOW
		glowcolor = texture2D(texture7,texcoord);
		ambientlight *= 1.0-glowcolor.w;
	#endif
	
	
	float specular = 0.0;
	vec4 shadowcolor = vec4(1.0);
	
	vec3 screencoord;
	screencoord = vec3((((gl_FragCoord.x+0.5)/buffersize.x)-0.5) * 2.0,(((-gl_FragCoord.y+0.5)/buffersize.y)+0.5) * 2.0 / (buffersize.x/buffersize.y),DepthToZPosition(depth));
	screencoord.x *= screencoord.z / camerazoom;
	screencoord.y *= -screencoord.z / camerazoom;

	if ((cnormal.x+cnormal.y+cnormal.z)>2.9) {
		attenuation=0.0;
		lightcolor=vec4(1);
		ambientlight=vec4(0);
	}
	
	//#ifdef __GLSL_CG_DATA_TYPES
	if (attenuation>0.0) {
	//#endif
			
		vec3 smoffset;
		mat3 lightmatrix;
		vec3 lightposition;
		
		shadowjitter_ /= 4.0;
		
		float shadowstage[4];
		shadowstage[0]=0.0;
		shadowstage[1]=0.0;
		shadowstage[2]=0.0;
		shadowstage[3]=0.0;

		float l=length(screencoord);

		#ifdef __GLSL_CG_DATA_TYPES
		if (l<shadowrange.z) {
		#endif
			
			vec3 smcoord;
			float zFar;
			float zNear;
			float shadowMapResolution;
			float psx;
			float psy;
			vec4 atihackshadowcolor=vec4(0.0);			

			//General parameters - these stay the same each stage
			zFar = gl_LightSource[0].linearAttenuation;
			zNear = 0.1;
			shadowMapResolution = gl_LightSource[0].quadraticAttenuation;
			psx = shadowsoftness / (shadowMapResolution * 2.0);
			psy = shadowsoftness / (shadowMapResolution * 2.0);
			//psy = shadowsoftness / (shadowMapResolution * 1.0);		

			fbool = float(l<shadowrange.z);
			shadowstage[2]=1.0*fbool;
			shadowstage[3]=1.0-fbool;
			
			fbool = float(l<shadowrange.y);
			shadowstage[0]=shadowstage[0]*(1.0-fbool) + 0.0*fbool;
			shadowstage[1]=shadowstage[1]*(1.0-fbool) + 1.0*fbool;
			shadowstage[2]=shadowstage[2]*(1.0-fbool) + 0.0*fbool;

			fbool = float(l<shadowrange.x);
			shadowstage[0]=shadowstage[0]*(1.0-fbool) + 1.0*fbool;
			shadowstage[1]=shadowstage[1]*(1.0-fbool) + 0.0*fbool;
			shadowstage[2]=shadowstage[2]*(1.0-fbool) + 0.0*fbool;
			
			#ifdef __GLSL_CG_DATA_TYPES

				lightmatrix	= lightmatrix0 * shadowstage[0] + lightmatrix1 * shadowstage[1] + lightmatrix2 * shadowstage[2];
				smoffset	= vec3(0.0,0.0,0.0) * shadowstage[0] + vec3(0.5,0.0,0.0) * shadowstage[1] + vec3(0.0,0.5,0.0) * shadowstage[2];

				lightposition 	= lightposition0 * shadowstage[0] + lightposition1 * shadowstage[1] + lightposition2 * shadowstage[2];

				smcoord = lightmatrix * ( lightposition - screencoord );
				linearshadowdepth = smcoord.z;
				smcoord.x /= -1.0;
				smcoord.y /= -1.0;
				smcoord.x += 0.5;
				smcoord.y += 0.5;
				smcoord.z = -(smcoord.z-zNear) / (zFar-zNear);
				smcoord.x *= 0.5;
				smcoord.y *= 0.5;
				smcoord += smoffset;
				
				//smcoord.y*=2.0;

				Include "ShadowLookup.frag"
				
				shadowcolor += max( (l - (shadowrange.z-10.0) ) / 10.0, 0.0 );
				shadowcolor = min(shadowcolor,1.0);
				
				#ifdef LW_LIGHTTEXTURE
					lighttint += max( (l - (shadowrange.z-10.0) ) / 10.0, 0.0 );
					lighttint = min(lighttint,1.0);
					lightcolor *= lighttint;
				#endif
				
			#else
				
				//To avoid visual artifacts, ATI cards must perform all texture lookups
				
				lightmatrix	= lightmatrix0;
				smoffset	= vec3(0.0,0.0,0.0);
				lightposition 	= lightposition0;
				
				smcoord = lightmatrix * ( lightposition - screencoord );
				smcoord.x /= -1.0;
				smcoord.y /= -1.0;
				smcoord.x += 0.5;
				smcoord.y += 0.5;
				smcoord.z = -(smcoord.z-zNear) / (zFar-zNear);
				smcoord.x *= 0.5;
				smcoord.y *= 0.5;
				smcoord += smoffset;
				
				Include "ShadowLookup.frag"
				
				shadowcolor += max( (l - (shadowrange.z-10.0) ) / 10.0, 0.0 );
				shadowcolor.x = min(shadowcolor.x,1.0);
				shadowcolor.y = min(shadowcolor.y,1.0);
				shadowcolor.z = min(shadowcolor.z,1.0);
				shadowcolor.w = min(shadowcolor.w,1.0);
				
				atihackshadowcolor += shadowcolor*shadowstage[0];
				#ifdef LW_LIGHTTEXTURE
					atihacklighttint += lighttint * shadowstage[0];
				#endif
				
				lightmatrix	= lightmatrix1;
				smoffset	= vec3(0.5,0.0,0.0);
				lightposition 	= lightposition1;
				shadowjitter_ /= 2.0;
				
				smcoord = lightmatrix * ( lightposition - screencoord );
				smcoord.x /= -1.0;
				smcoord.y /= -1.0;
				smcoord.x += 0.5;
				smcoord.y += 0.5;
				smcoord.z = -(smcoord.z-zNear) / (zFar-zNear);
				smcoord.x *= 0.5;
				smcoord.y *= 0.5;
				smcoord += smoffset;
				
				Include "ShadowLookup.frag"
				
				shadowcolor += max( (l - (shadowrange.z-10.0) ) / 10.0, 0.0 );
				shadowcolor = min(shadowcolor,1.0);
				
				atihackshadowcolor += shadowcolor*shadowstage[1];
				#ifdef LW_LIGHTTEXTURE
					atihacklighttint += lighttint * shadowstage[1];
				#endif
				
				lightmatrix	= lightmatrix2;
				smoffset	= vec3(0.0,0.5,0.0);
				lightposition 	= lightposition2;
				shadowjitter_ /= 2.0;
				
				smcoord = lightmatrix * ( lightposition - screencoord );
				smcoord.x /= -1.0;
				smcoord.y /= -1.0;
				smcoord.x += 0.5;
				smcoord.y += 0.5;
				smcoord.z = -(smcoord.z-zNear) / (zFar-zNear);
				smcoord.x *= 0.5;
				smcoord.y *= 0.5;
				smcoord += smoffset;
				
				Include "ShadowLookup.frag"
				
				shadowcolor += max( (l - (shadowrange.z-10.0) ) / 10.0, 0.0 );
				shadowcolor = min(shadowcolor,1.0);
				
				atihackshadowcolor += shadowcolor*shadowstage[2];
				#ifdef LW_LIGHTTEXTURE
					atihacklighttint += lighttint * shadowstage[2];
				#endif
				
				atihackshadowcolor += shadowstage[3];
				#ifdef LW_LIGHTTEXTURE
					atihacklighttint += shadowstage[3];
				#endif
				
				shadowcolor=atihackshadowcolor;
				
				#ifdef LW_LIGHTTEXTURE
					lighttint = atihacklighttint;
					lightcolor *= lighttint;
				#endif
				
			#endif
			
			#ifdef __GLSL_CG_DATA_TYPES
			}
			#endif

		#ifdef LW_SPECULAR			
			vec3 R = -reflect(vec3(lightdirection.x,lightdirection.y,-lightdirection.z), vec3(normal.x,normal.y,-normal.z));
			specular = pow(clamp(dot(R, normalize(-screencoord) ),0.0,1.0),gloss) * attenuation * gl_LightSource[0].diffuse.w * materialspecular;
		#endif
		
	//#ifdef __GLSL_CG_DATA_TYPES
	}
	//#endif
	
	#ifdef LW_LIGHTTEXTURE
	//lightcolor *= texture2D(texture4,vec2( dot(mappingaxis[0].xyz,screencoord) + mappingaxis[0].w, dot(mappingaxis[1].xyz,screencoord) + mappingaxis[1].w ));
	#endif
	
	vec4 outcolor;
	
	//diffuse = vec4(1);

	fbool = float(depth==1.0);
	#ifdef LW_WRITEBACKGROUND
		outcolor = 1.0 * (diffuse * (lightcolor * shadowcolor + ambientlight) + specular*lightcolor * shadowcolor) * (1.0-fbool) + (diffuse * fbool);
	#else
		outcolor = 1.0 * (diffuse * (lightcolor * shadowcolor + ambientlight) + specular*lightcolor * shadowcolor) * (1.0-fbool);
	#endif

	#ifndef __GLSL_CG_DATA_TYPES
		if (attenuation==0.0) {
			//outcolor = diffuse;
		}
	#endif	
	
	#ifdef LW_GLOW
	outcolor += glowcolor;
	#endif
	
	outcolor.x = max(0.0,outcolor.x);
	outcolor.y = max(0.0,outcolor.y);
	outcolor.z = max(0.0,outcolor.z);
	
	gl_FragData[0] = outcolor;
	//gl_FragData[0]=cnormal;
}


