#extension GL_ARB_draw_buffers : enable

#define shadowsoftness 1.0

uniform sampler2D texture0;//color
uniform sampler2D texture1;//depth
uniform sampler2D texture2;//normal
uniform sampler2D texture3;//specular/gloss
uniform sampler2DShadow texture4;//shadowmap
uniform sampler2D texture6;//light texture
uniform sampler2DShadow texture8;// color shadowmap
uniform sampler2D texture5;//noise
uniform float cameraaspect;
uniform float camerafov;
uniform float camerazoom;
uniform vec2 camerarange;
uniform mat3 lightmatrix0;
uniform vec3 lightdirection;
uniform vec2 coneanglecos;
uniform vec3 lightposition0;
uniform vec2 buffersize;
uniform float bufferaspect;
uniform vec2 shadowmapsize;
uniform float rnd;
uniform float shadowjitter;

varying vec4 fake_FragCoord1;

Include "DepthToZPosition.frag"
Include "shadow2D4.frag"

void main(void) {

	#ifdef LW_LIGHTTEXTURE
		vec4 coloredshadowcolor = vec4(1);
	#endif

	vec2 noise;
	float sw;
	float sh;
	float shadowjitter_=shadowjitter;
	vec2 texCoord = gl_FragCoord.xy/buffersize;
	vec4 sample3 = texture2D(texture3,texCoord);
	float materialspecular = sample3.x;
	float gloss = sample3.y * 16.0;	
	vec4 lightcolor = gl_LightSource[0].diffuse;
	float depth = texture2D(texture1,texCoord).x;
	float attenuation = 1.0;
	vec4 diffuse = texture2D(texture0,texCoord);
	vec4 cnormal = texture2D(texture2,texCoord);
	vec3 normal = cnormal.xyz * 2.0 - 1.0;
	vec3 screencoord;
	vec3 smcoord;
	float specular=0.0;
	vec3 normalizedlightdir;
	vec4 shadowcolor = vec4(1.0);
	vec3 lightdir;
	float lightdirlen;
	float falloff;
	vec3 olightdir;
	vec4 lighttint;
	
	screencoord = vec3(((gl_FragCoord.x/buffersize.x)-0.5) * 2.0,((-gl_FragCoord.y/buffersize.y)+0.5) * 2.0 / (buffersize.x/buffersize.y),DepthToZPosition( depth ));
	screencoord.x *= screencoord.z / camerazoom;
	screencoord.y *= -screencoord.z / camerazoom;
	
	lightdir = screencoord - lightposition0;		
	//attenuation *= clamp( dot( normal,normalize(vec3(-lightdir.x,-lightdir.y,lightdir.z))),0.0,1.0 );	

	attenuation *= clamp( dot( normal,normalize(vec3(-lightdir.x,-lightdir.y,lightdir.z))),0.0,1.0 );
	lightdirlen = length( lightdir );
	
	falloff = clamp( lightdirlen/gl_LightSource[0].linearAttenuation ,0.0,1.0);
	#ifdef LW_INVERSESQUAREFALLOFF	
		attenuation *= 1.0 - falloff*falloff;
	#else
		attenuation *= 1.0 - falloff;
	#endif
	
	olightdir=lightdir;		
	normalizedlightdir = lightdir/lightdirlen;
	
	if ((cnormal.x+cnormal.y+cnormal.z)==3.0) { discard; }
//	if (attenuation<0.001) { discard; }
	
	lightdir = lightmatrix0 * lightdir;
	
	float zFar = gl_LightSource[0].linearAttenuation;
	float zNear = 0.1;
	float a = zFar / ( zFar - zNear );
	float b = zFar * zNear / ( zNear - zFar );
	float shadowMapResolution = gl_LightSource[0].quadraticAttenuation;
	float psx = shadowsoftness / (shadowMapResolution * 4.0);
	float psy = shadowsoftness / (shadowMapResolution * 2.0);
	
	smcoord=lightdir;
	smcoord.x /= -smcoord.z/0.5;
	smcoord.y /= -smcoord.z/0.5;
	
	if (smcoord.x>0.5) { discard; }
	if (smcoord.x<-0.5) { discard; }
	if (smcoord.y>0.5) { discard; }
	if (smcoord.y<-0.5) { discard; }
	if (smcoord.z<0.0) { discard; }
	
	smcoord.x += 0.5;
	smcoord.y += 0.5;
	smcoord.z = a + b / smcoord.z;
	
	Include "ShadowLookup.frag"
	
	#ifdef LW_LIGHTTEXTURE
		lightcolor *= lighttint;//texture2D(texture6,vec2(smcoord.x,-smcoord.y));
	#endif
	
	#ifdef LW_SPECULAR
		vec3 R = -reflect(-(olightdir/lightdirlen), vec3(normal.x,normal.y,-normal.z));
		specular = pow(clamp(dot(R, normalize(-screencoord) ),0.0,1.0),gloss) * attenuation * gl_LightSource[0].diffuse.w * materialspecular;
	#endif
	
	gl_FragData[0] = (lightcolor * diffuse * attenuation * shadowcolor) + (specular * shadowcolor);
}


