#extension GL_ARB_draw_buffers : enable

//==========================================
// Spot Light
//==========================================

uniform sampler2D texture0;//color
uniform sampler2D texture1;//depth
uniform sampler2D texture2;//normal
uniform sampler2D texture3;//specular/gloss
uniform sampler2DShadow texture4;//shadowmap
uniform sampler2D texture6;//light texture
uniform sampler2D texture5;//noise
uniform sampler2DShadow texture8;//secondary depth map
uniform float cameraaspect;
uniform float camerafov;
uniform float camerazoom;
uniform vec2 camerarange;
uniform mat3 lightmatrix0;
uniform vec3 lightdirection;
uniform vec2 coneanglecos;
uniform vec3 lightposition0;
uniform vec2 buffersize;
uniform float bufferaspect;
//uniform float shadowsoftness;
#define shadowsoftness 1.0
uniform vec2 shadowmapsize;

uniform float rnd;

varying vec4 fake_FragCoord1;

uniform float shadowjitter;

Include "DepthToZPosition.frag"
Include "shadow2D4.frag"

/*
vec4 shadow2D4(in sampler2DShadow texture, in vec3 texcoord, in float width, in float height) {
	float alpha = fract(texcoord.x * width - 0.5);
	float beta = fract(texcoord.y * height - 0.5);
	
	vec3 st_00 = vec3(texcoord.x-0.5/width,texcoord.y-0.5/height,texcoord.z);
	vec3 st_10 = vec3(texcoord.x+0.5/width,texcoord.y-0.5/height,texcoord.z);
	vec3 st_01 = vec3(texcoord.x-0.5/width,texcoord.y+0.5/height,texcoord.z);
	vec3 st_11 = vec3(texcoord.x+0.5/width,texcoord.y+0.5/height,texcoord.z);
	
	return (1.0 - alpha)*(1.0 - beta) * shadow2D(texture, st_00)
		+ alpha * (1.0 - beta) * shadow2D(texture, st_10)
		+ (1.0 - alpha)*beta * shadow2D(texture, st_01)
		+ alpha * beta * shadow2D(texture, st_11);
}
*/

void main(void) {
	
	#ifdef LW_LIGHTTEXTURE
		vec4 coloredshadowcolor = vec4(1);
	#endif
	
	vec2 noise;
	float sw;
	float sh;
	float shadowjitter_=shadowjitter;
	vec4 lighttint;
	
	//vec2 fake_FragCoord = ((fake_FragCoord1.xy/fake_FragCoord1.w)*0.5+0.5)*buffersize;

	//vec2 texCoord = vec2( fake_FragCoord.x/buffersize.x, fake_FragCoord.y/buffersize.y);

	vec2 texCoord = gl_FragCoord.xy/buffersize;


	vec4 sample3 = texture2D(texture3,texCoord);
	float materialspecular = sample3.x;
	float gloss = sample3.y * 16.0;	


	vec4 lightcolor = gl_LightSource[0].diffuse;

	float depth = texture2D(texture1,texCoord).x;

	float attenuation = 1.0;
	
	vec4 diffuse = texture2D(texture0,texCoord);
	vec4 cnormal = texture2D(texture2,texCoord);
	vec3 normal = cnormal.xyz * 2.0 - 1.0;
	
	//float gloss = diffuse.w * 16.0;

	vec3 screencoord;
	screencoord = vec3(((gl_FragCoord.x/buffersize.x)-0.5) * 2.0,((-gl_FragCoord.y/buffersize.y)+0.5) * 2.0 / (buffersize.x/buffersize.y),DepthToZPosition( depth ));
	screencoord.x *= screencoord.z / camerazoom;
	screencoord.y *= -screencoord.z / camerazoom;
		
	vec3 lightdir = screencoord - lightposition0;		
	attenuation *= clamp( dot( normal,normalize(vec3(-lightdir.x,-lightdir.y,lightdir.z))),0.0,1.0 );	
	float lightdirlen = length( lightdir );
	//attenuation *= 1.0 - clamp( lightdirlen/gl_LightSource[0].linearAttenuation ,0.0,1.0);

	float falloff = clamp( lightdirlen/gl_LightSource[0].linearAttenuation ,0.0,1.0);
	#ifdef LW_INVERSESQUAREFALLOFF	
		attenuation *= 1.0 - falloff*falloff;
	#else
		attenuation *= 1.0 - falloff;
	#endif
	
	vec4 shadowcolor = vec4(1.0);
	
	vec3 olightdir=lightdir;	
	
	vec3 normalizedlightdir;
	normalizedlightdir = lightdir/lightdirlen;
	
	float anglecos=dot(lightdirection,normalize(normalizedlightdir));
	attenuation *= clamp((coneanglecos.y-anglecos)/(coneanglecos.y-coneanglecos.x),0.0,1.0);
	
	float specular=0.0;
	
	if ((cnormal.x+cnormal.y+cnormal.z)==3.0) {
		attenuation=0.0;
		depth=1.0;
		discard;
	}
	
	if (attenuation>0.001) {
		
		lightdir = lightmatrix0 * lightdir;
		
		//=============================================
		// Shadow
		//=============================================
		
		vec3 smcoord;
		
		float zFar = gl_LightSource[0].linearAttenuation;
		float zNear = 0.1;
		float a = zFar / ( zFar - zNear );
		float b = zFar * zNear / ( zNear - zFar );
		
		float shadowMapResolution = gl_LightSource[0].quadraticAttenuation;
		float psx = shadowsoftness / (shadowMapResolution * 4.0);
		float psy = shadowsoftness / (shadowMapResolution * 2.0);
		
		smcoord=lightdir;
		smcoord.x /= -smcoord.z/0.5;
		smcoord.y /= -smcoord.z/0.5;
		smcoord.x += 0.5;
		smcoord.y += 0.5;
		smcoord.z = a + b / smcoord.z;
		
		Include "ShadowLookup.frag"
		
		#ifdef LW_LIGHTTEXTURE
			lightcolor *= lighttint;
			//lightcolor *= texture2D(texture6,vec2(smcoord.x,-smcoord.y));
		#endif
		
		//=============================================
		//
		//=============================================
		
		
		//=============================================
		//Specular reflection
		//=============================================
		
		#ifdef LW_SPECULAR
		//if (shadowcolor>0.001) {
			vec3 R = -reflect(-(olightdir/lightdirlen), vec3(normal.x,normal.y,-normal.z));
			specular = pow(clamp(dot(R, normalize(-screencoord) ),0.0,1.0),gloss) * attenuation * gl_LightSource[0].diffuse.w * materialspecular;
		//}
		#endif
		
		//=============================================
		//
		//=============================================
	

		gl_FragData[0] = lightcolor * diffuse * attenuation * shadowcolor + specular * shadowcolor;
		//gl_FragData[1]=vec4(0.0);
		//gl_FragData[2] = vec4(specular*shadowcolor*4.0*diffuse) * gl_LightSource[0].diffuse;

//gl_FragData[0]=vec4(1);
		
		}
	else {
		discard;
	}
	
}


