#extension GL_ARB_draw_buffers : enable

uniform vec3 cameraposition;
uniform vec2 buffersize;
uniform vec2 camerarange;

include "abstract::greyscale.txt"
include "abstract::DepthToZPosition.frag"

#ifdef LW_DETAIL
	uniform sampler2D LW_DETAIL;
#endif

#ifdef LW_DIFFUSE
	uniform sampler2D LW_DIFFUSE;
#endif

#ifdef LW_DIFFUSE2
	uniform sampler2D LW_DIFFUSE2;
#endif

#ifdef LW_SPECULARMAP
	uniform sampler2D LW_SPECULARMAP;
#endif

#ifdef LW_BUMPMAP
	uniform sampler2D LW_BUMPMAP;
#endif

#ifdef LW_BUMPMAP2
	uniform sampler2D LW_BUMPMAP2;
#endif

#ifdef LW_BLOOM
	uniform sampler2D LW_BLOOM;
#endif

#ifdef LW_CUBEMAP
	uniform samplerCube LW_CUBEMAP;
	varying vec3 cubemapdir;
	varying vec3 cubemapnormal;
#endif

#ifdef LW_GIMAP
	uniform sampler2D LW_GIMAP;
#endif

#ifdef LW_PARALLAXMAP
	uniform sampler2D LW_PARALLAXMAP;
	varying vec3 eyevec;
#endif

#ifdef LE_REFRACTION
	uniform sampler2D LE_REFRACTION;
	uniform sampler2D LE_DEPTHBUFFER;
	uniform float refractionstrength = 0.01;
#endif

#ifdef LW_POMMAP
	vec3 vLightTS=vec3(0.577,0.577,0.577);
	varying vec3 eyevec;
	float depthP = 0.01;
	float nMinSamples = 20;
	float nMaxSamples = 50;	
#endif

#ifdef LW_MESHLAYER
	varying float vegetationfade;
#endif

#ifdef LW_ALPHABLEND
	uniform sampler2D LW_ALPHABLEND_INCOMINGCOLOR;
	uniform sampler2D LW_ALPHABLEND_INCOMINGNORMAL;
#endif

varying vec3 vertexposition;
varying vec3 T,B,N;
varying vec2 texcoord0;
varying vec2 texcoord1;
varying vec4 modelvertex;
varying vec4 fragcolor;

float fOcclusionShadow = 1.0;

uniform sampler2D texture14;
uniform float terrainsize;
uniform vec3 terrainscale;
uniform float bumpscale;
uniform float specular;
uniform float gloss;

//Terrain color map
uniform sampler2D texture12;

void main(void) {
	
	vec4 diffuse = fragcolor;
	vec3 normal;
	float shininess = 0.0;
	vec2 texcoord=texcoord0;// only use this because of parallax mapping
	float selfillumination = 0.0;	

	vec2 terraincoord;
	float terrainresolution;
	
	#ifdef LW_VERTEXGI
		diffuse = vec4(1);
	#endif
	
	#ifdef LW_PARALLAXMAP
		texcoord += (diffuse.w * 0.04 - 0.036) * normalize(eyevec).xy;
		//texcoord += (texture2D(LW_DIFFUSE,texcoord).w * 0.04 - 0.036) * normalize(eyevec).xy;
	#endif
	
	#ifdef LW_POMMAP
		// for POM, the heightmap is in the alpha of the diffuse so save ur diffuse with DXT5 I chose this because the alpha of DXT5 is higher precision
		Include "POM.frag"
	#endif

	#ifdef LW_DIFFUSE2
		vec4 diffuse2 = texture2D(LW_DIFFUSE2,texcoord);
		diffuse = vec4(1);
	#endif	
	
	#ifdef LW_DIFFUSE
		diffuse *= texture2D(LW_DIFFUSE,texcoord);//*fOcclusionShadow;
	#endif

	#ifdef LW_ALPHATEST
		if (diffuse.w<0.5) {
			discard;
		}
	#endif


	normal = N;
	
	#ifdef LW_BUMPMAP

		#ifdef LW_TERRAINNORMALS
			//Use terrain normals
			terraincoord=vec2(vertexposition.x,-vertexposition.z) / terrainsize + 0.5;
			terrainresolution = terrainsize / terrainscale.x;
			terraincoord += 0.5 / terrainresolution;
			vec3 worldNormal = ((texture2D(texture14,terraincoord).xyz - 0.5) * 2.0).xyz;
			normal = normalize(gl_NormalMatrix*worldNormal);
		#else
			vec4 bumpcolor = texture2D(LW_BUMPMAP,texcoord);
			normal = bumpcolor.xyz * 2.0 - 1.0;
		#endif
		
		#ifdef LW_DETAIL
			normal += texture2D(LW_DETAIL,texcoord * 4.0).xyz * 2.0 - 1.0;
		#endif
		normal.z /= bumpscale;
		normal = T * normal.x + B * normal.y + N * normal.z;
		normal = normalize(normal);
		#ifdef LW_SPECULAR
			shininess = bumpcolor.a*specular;//*fOcclusionShadow
		#endif
		#ifdef LW_SPECULARMAP
			shininess = texture2D(LW_SPECULARMAP,texcoord).x*specular;//*fOcclusionShadow
		#endif
	
	#else
		normal=normalize(normal);
	#endif

	#ifdef LW_BUMPMAP2
		vec3 normal2;
		float shininess2;
		
		vec4 bumpcolor2 = texture2D(LW_BUMPMAP2,texcoord);
		normal2 = bumpcolor2.xyz * 2.0 - 1.0;
		
		normal2.z /= bumpscale;
		normal2 = T * normal2.x + B * normal2.y + N * normal2.z;
		normal2 = normalize(normal2);
		#ifdef LW_SPECULAR
			shininess2 = bumpcolor2.a*specular;
		#endif
		
	#endif	
	
	#ifdef LW_TERRAINNORMALS
		#ifndef LW_BUMPMAP
			//Use terrain normals
			terraincoord=vec2(vertexposition.x,-vertexposition.z) / terrainsize + 0.5;
			terrainresolution = terrainsize / terrainscale.x;
			terraincoord.x -= 0.5 / terrainresolution;
			//vec4 normsample=((texture2D(texture14,terraincoord).xyz - 0.5) * 2.0).xyz;
			vec3 worldNormal = ((texture2D(texture14,terraincoord).xyz - 0.5) * 2.0).xyz;
			normal = normalize(gl_NormalMatrix*worldNormal);
			//shininess = normsample.w;
		#endif
	#endif

	#ifdef LW_TERRAINCOLOR
		//Use terrain color
		terraincoord=vec2(vertexposition.x,-vertexposition.z) / terrainsize + 0.5;
		terrainresolution = terrainsize / terrainscale.x;
		terraincoord.x -= 0.5 / terrainresolution;
		//terraincoord.y += 0.5 / terrainresolution;
		vec4 terraincolor = texture2D(texture12,terraincoord);
		diffuse = vec4( greyscale(diffuse.xyz) * 2.0 * terraincolor.xyz,diffuse.w);

		#ifdef LW_MESHLAYER
			float temp_w=diffuse.w;
			diffuse = diffuse * (1.0-vegetationfade) + terraincolor * vegetationfade;
			diffuse.w=temp_w;
		#endif

		shininess = terraincolor.w;
	#endif
	
	#ifdef LE_REFRACTION
		diffuse.a=0.25;
		vec4 refractionvector = vec4( gl_FragCoord.x/buffersize.x, gl_FragCoord.y/buffersize.y, gl_FragCoord.z, 1.0 );
		vec4 refractionvector2 = refractionvector + refractionstrength * vec4(normal,0.0);		
		if (gl_FragCoord.z<DepthToZPosition(texture2DProj(LE_DEPTHBUFFER,refractionvector2).x)) {
			refractionvector=refractionvector2;
		}
		vec4 transparency = texture2DProj(LE_REFRACTION,refractionvector);
		diffuse = transparency * diffuse;
	#endif

	#ifdef LW_BUMPMAP2
		shininess = fragcolor.r * shininess + (1.0-fragcolor.r) * shininess2;
		normal = fragcolor.r * normal + (1.0-fragcolor.r) * normal2;
	#endif
	
	vec3 adjustednormal = normal*0.5+0.5;
	float adjustedgloss = gloss;
	
	shininess=clamp(shininess,0.0,1.0)*0.5;
	


	#ifdef LW_DIFFUSE2
		diffuse = fragcolor.r * diffuse + (1.0-fragcolor.r) * diffuse2;
	#endif	

    #ifdef LW_CUBEMAP
		vec3 cubecoord = reflect( normalize( modelvertex.xyz - cameraposition ), normal * gl_NormalMatrix );
		diffuse = vec4( textureCube(LW_CUBEMAP,cubecoord).xyz, diffuse.w);
    #endif
    
	//Diffuse
	gl_FragData[0] = diffuse;	

	//Normal
	#ifdef LW_FULLBRIGHT
		gl_FragData[1] = vec4(1.0,1.0,1.0,diffuse.w);
	#else
		gl_FragData[1] = vec4(adjustednormal,diffuse.w);
	#endif
	
	//Bloom
	#ifdef LW_BLOOM
		vec4 bloomcolor = texture2D(LW_BLOOM,texcoord) * fragcolor;
		gl_FragData[3] = bloomcolor;
		gl_FragData[3].w = 0;
	#else
		#ifdef LW_GIMAP
			vec4 gicolor = texture2D(LW_GIMAP,vec2(texcoord1.x,1.0-texcoord1.y));
			
			//We're going to clamp the results to prevent overbrightening when the lightmap environment doesn't match the dynamic lighting
			//float gilum = gicolor.r * 0.3 + gicolor.g * 0.59 + gicolor.b * 0.11;
			//float giscale = clamp(gilum,0,0.1) / gilum;
			//gicolor.x *= giscale;
			//gicolor.y *= giscale;
			//gicolor.z *= giscale;
			gicolor *= diffuse;
			gicolor.w = 1.0;
			gl_FragData[3] = gicolor;
		#else
			#ifdef LW_VERTEXGI
				gl_FragData[3] = fragcolor * diffuse;
				gl_FragData[3].w = 1.0;
			#else
				gl_FragData[3] = vec4(0);
			#endif
		#endif
	#endif

	//Modify depth output for shadows
	#ifdef LW_POMMAP
		float z = DepthToZPosition(gl_FragCoord.z);
		z = z -diffuse.a * fparallaxlength * depthP;
		gl_FragDepth = ZPositionToDepth( z );
	#endif
	
	gl_FragData[2]=vec4(shininess,gloss,0.0,diffuse.w);
	
	/*
	float c;
	float temp;
	temp=adjustednormal.x*100.0;
	c=int(temp);
	temp=adjustednormal.y*100.0;
	c+=int(temp)/100.0;
	temp=adjustednormal.z*100.0;
	c+=int(temp)/100.0/100.0;
	gl_FragData[1].x = c;
	*/
}
