Include "MeshMatrix.vert"

uniform float apptime;
uniform vec2 buffersize;
uniform float meshlayerrange;
uniform float meshlayerheight;
uniform vec3 cameraposition;
uniform float camerazoom;
uniform float vegetationviewdistance;
uniform float vegetationheight;

varying vec3 T,B,N;
varying vec2 texcoord0;
varying vec2 texcoord1;
varying vec3 vertexposition;
varying vec4 modelvertex;
varying vec4 projectionspace;
varying vec4 refractionspace;
varying vec4 fragcolor;

#ifdef LW_CUBEMAP
	varying vec3 cubemapdir;
	varying vec3 cubemapnormal;
#endif

#ifdef LW_PARALLAXMAP
	varying vec3 eyevec;
#endif

#ifdef LW_MESHLAYER
	varying float vegetationfade;
#endif

void main(void) {
	
	vec4 vertexcameraposition;
	
	Include "GetMatrix.vert"
	
	fragcolor = gl_Color*color;

	modelvertex = gl_Vertex;
	
	#ifdef LW_SKIN
		fragcolor=color;
		mat  = GetAnimMatrix( int(boneindices[0]) ) * gl_Color[0];
		mat += GetAnimMatrix( int(boneindices[1]) ) * gl_Color[1];
		mat += GetAnimMatrix( int(boneindices[2]) ) * gl_Color[2];
		mat += GetAnimMatrix( int(boneindices[3]) ) * gl_Color[3];
		/*
		mat = animmatrix[ int(boneindices[0]) ] * gl_Color[0];
		mat += animmatrix[ int(boneindices[1]) ] * gl_Color[1]; 
		mat += animmatrix[ int(boneindices[2]) ] * gl_Color[2];
		mat += animmatrix[ int(boneindices[3]) ] * gl_Color[3];
		*/
	#endif
	
	mat3 nmat = mat3(mat[0].xyz,mat[1].xyz,mat[2].xyz);
		
	#ifdef LW_MESHLAYER
		mat[3].y=0.0;
	#endif
	
	modelvertex = mat * modelvertex;
	
	#ifdef LW_MESHLAYER
		//Align to terrain height
		modelvertex.y += GetTerrainHeight(modelvertex.xz);
		//Sink into ground with distance
		float mld = length(modelvertex.xyz - cameraposition )/vegetationviewdistance;
		//mld += abs(mat[3].x-int(mat[3].x)) * 0.25;
		vegetationfade=mld;
		mld = clamp( (mld-0.75)*4.0, 0.0, 1.0 );
		modelvertex.y -= mld * vegetationheight;
	#endif

	#ifdef LW_SWAY
		float seed = mod(apptime / 100.0 * 0.25,360.0);
		seed += mat[3].x*33.0 + mat[3].y*67.8 + mat[3].z*123.5;
		seed += gl_Vertex.x + gl_Vertex.y + gl_Vertex.z;
		vec4 movement = vec4( vec3( gl_Color.x * gl_Normal * LW_SWAY * (sin(seed)+0.25*cos(seed*5.2+3.2)) ),0.0);		
		modelvertex += movement;
		fragcolor = color;
	#endif

	vertexposition = modelvertex.xyz;
	vertexcameraposition = gl_ModelViewProjectionMatrix * modelvertex;		

	texcoord0=gl_MultiTexCoord0.xy;
	texcoord1=gl_MultiTexCoord1.xy;
	
	#ifdef LW_CUBEMAP
		cubemapdir = normalize( modelvertex.xyz - cameraposition );
		cubemapnormal = normalize(nmat * gl_Normal).xyz;
		//cubecoord = reflect( normalize( modelvertex.xyz - cameraposition ) , normalize(nmat * gl_Normal).xyz );
	#endif
	
	nmat = gl_NormalMatrix * nmat;
	
	N = normalize( nmat * gl_Normal );
	#ifdef LW_BUMPMAP
		T = normalize( nmat * gl_MultiTexCoord2.xyz );		
		B = normalize( nmat * gl_MultiTexCoord3.xyz );
	#endif
	
	gl_Position = vertexcameraposition;
	
	#ifdef __GLSL_CG_DATA_TYPES
		gl_ClipVertex = gl_ModelViewMatrix * modelvertex;
	#endif

	#ifdef LW_PARALLAXMAP
		modelvertex = gl_ModelViewMatrix * modelvertex;
		mat3 TBN_Matrix;
	    	TBN_Matrix[0] = T;
		TBN_Matrix[1] = B;
		TBN_Matrix[2] = N;
		eyevec = vec3(-modelvertex) * TBN_Matrix;
//		modelvertex = gl_ModelViewMatrix * modelvertex;
//		EyeVec = vec3(dot(T,-modelvertex), dot(B,-modelvertex), dot(N,-modelvertex));
	#endif
	
}
