#extension GL_ARB_draw_buffers : enable

#define amplitude 4.0
#define wavesize 20.0

uniform sampler2D texture0;// reflection color buffer
uniform sampler2D texture1;// normal map
uniform sampler2D texture2;// refraction depth buffer
uniform sampler2D texture3;// surf texture
uniform sampler2D texture4;// refraction texture
uniform sampler2D texture5;// wobble shader
uniform sampler2D texture6;//heightmap
uniform sampler2D texture7;//wave normalmap
uniform sampler2D texture8;//depth gradient
uniform vec2 fogrange=vec2(0.0,300.0);
uniform vec4 fogcolor=vec4(0.5,0.5,0.5,1);
uniform sampler2D texture9;//

uniform float apptime;
uniform vec2 buffersize;
uniform vec2 camerarange;
uniform vec3 sunposition;// = vec3(1000.0,1000.0,1000.0);
uniform vec3 cameraposition;
uniform float waterdepth=20.0;
uniform float wobbleamplitude=0.025;
uniform float camerazoom=1.0;

uniform float MajorWaveSpeed = 0.01;
uniform float MajorWaveScale = 0.005;
uniform float MinorWaveSpeed = 0.01;
uniform float MinorWaveScale = 0.005;

uniform vec3 SunDirection=vec3(0.577,-0.577,0.577);

varying vec3 T,B,N;
varying vec4 projectspace;
varying vec4 refractionspace;
varying vec2 texcoord;
varying vec4 modelvertex;
varying vec4 fragcolor;


float DepthToZPosition(in float depth) {
	return camerarange.x / (camerarange.y - depth * (camerarange.y - camerarange.x)) * camerarange.y;
}

#define COS22 0.924
#define SIN22 0.383


void main() {

	float appseconds = apptime / 1000.0;
	
	//vec3 fake_FragCoord;
	//fake_FragCoord.x = ((fake_FragCoord1.x/fake_FragCoord1.w)*0.5+0.5)*buffersize.x;
	//fake_FragCoord.y = ((fake_FragCoord1.y/fake_FragCoord1.w)*0.5+0.5)*buffersize.y;
	//fake_FragCoord.z = 0.5*fake_FragCoord1.z/fake_FragCoord1.w+0.5;
	
	vec2 wobble = ( texture2D(texture5,vec2(texcoord.x/32.0 + apptime / 50000.0,texcoord.y/32.0)).xy-0.5 ) * 0.1;
	
	float waterheight;
	vec3 normal;
	vec4 bumpcolor;

	/*
	bumpcolor  = texture2D(texture1,vec2(texcoord.y/wavesize,		texcoord.x/wavesize + apptime / 50000.0));
	bumpcolor += texture2D(texture1,vec2(texcoord.y/wavesize+0.5,		texcoord.x/wavesize - apptime / 50000.0));
	bumpcolor += texture2D(texture1,vec2(texcoord.y/wavesize/2.0,		texcoord.x/wavesize/2.0 + apptime / 50000.0 / 2.0 ));
	bumpcolor += texture2D(texture1,vec2(texcoord.y/wavesize/2.0+0.5,	texcoord.x/wavesize/2.0 - apptime / 50000.0 / 2.0 ));
	*/

	//bumpcolor += texture2D(texture1,vec2(texcoord.y/wavesize,		texcoord.x/wavesize + apptime / 50000.0));
	//bumpcolor += texture2D(texture1,vec2(texcoord.y/wavesize+0.5,		texcoord.x/wavesize - apptime / 50000.0));
	//bumpcolor /= 4.0;

	//waterheight=bumpcolor.w;
	//normal = bumpcolor.xyz - 0.5;
	//normal.z /= 2.0;
	//normal = normalize(normal);
	//normal=vec3(0,1,0);
	
	vec2 oceancoord;
	oceancoord.x = modelvertex.x * MajorWaveScale;
	oceancoord.y = modelvertex.z * MajorWaveScale - appseconds * MajorWaveSpeed;
	normal  = texture2D(texture1,oceancoord).xyz * 2.0 - 1.0;

	oceancoord.x = (-COS22*modelvertex.x+SIN22*modelvertex.z)*MajorWaveScale;
	oceancoord.y = (-SIN22*modelvertex.x-COS22*modelvertex.z)*MajorWaveScale - (-appseconds*MajorWaveSpeed);
	normal += texture2D(texture1,oceancoord).xyz * 2.0 - 1.0;

	vec3 onormal=normalize(normal);

	oceancoord.x = modelvertex.x * MinorWaveScale;
	oceancoord.y = modelvertex.z * MinorWaveScale - appseconds * MinorWaveSpeed;
	normal += texture2D(texture1,oceancoord).xyz * 2.0 - 1.0;
	
	oceancoord.x = (-COS22*modelvertex.x+SIN22*modelvertex.z)*MinorWaveScale;
	oceancoord.y = (-SIN22*modelvertex.x-COS22*modelvertex.z)*MinorWaveScale - (-appseconds*MinorWaveSpeed);
	normal += texture2D(texture1,oceancoord).xyz * 2.0 - 1.0;

	bumpcolor =  texture2D(texture5,vec2(texcoord.x/2.0,texcoord.y/2.0 + apptime / 20000.0));
	bumpcolor += texture2D(texture5,vec2(texcoord.y/2.0,texcoord.x/2.0 + apptime / 20000.0));
	bumpcolor /= 2.0;

	//Surface floating stuff
	float surfacedetail;
	#define detailfreq 5.0
	surfacedetail = texture2D(texture9,vec2(modelvertex.x/detailfreq + bumpcolor.x * 0.025 + cos(apptime/10000.0)/10.0,0.5+modelvertex.z * 0.05 + modelvertex.z/detailfreq + sin(apptime/10000.0)/10.0)).x;
	surfacedetail += texture2D(texture9,vec2(0.5+modelvertex.z/detailfreq + bumpcolor.y * 0.025 - sin(apptime/10000.0)/10.0,modelvertex.x * 0.05 + modelvertex.x/detailfreq - cos(apptime/10000.0)/10.0)).x;
	surfacedetail /= 2.0;
	
	

	float gloss;// = (0.5 + 0.5 * texture2D(texture9,modelvertex.xz/10.0).x) * 1.0;

	normal.z = normal.z / 3.0 * (0.5+0.5*surfacedetail);
	//normal.z *= gloss;
	normal = normalize(normal);
	
	//vec4 watercolor;
	//watercolor= texture2D(texture6,vec2(wobble.x + texcoord.x/wavesize,		wobble.y + texcoord.y/wavesize + apptime / 20000.0));
	//watercolor+= texture2D(texture6,vec2(wobble.x + texcoord.x/wavesize+0.5,	wobble.y + texcoord.y/wavesize - apptime / 20000.0));	
	//watercolor+= texture2D(texture6,vec2(wobble.x + texcoord.y/wavesize/8.0,	wobble.y + texcoord.x/wavesize/8.0 + apptime / 50000.0));
	//watercolor+= texture2D(texture6,vec2(wobble.x + texcoord.y/wavesize/8.0+0.5,	wobble.y + texcoord.x/wavesize/8.0 - apptime / 50000.0));
	//watercolor/= 2.0;
	
	
	
	// Calculate lighting
	vec3 lightvector = normalize(sunposition-modelvertex.xyz);
	float brightness = 1.0 * max(0.0,dot(normal.xzy,-lightvector));
	
	brightness = brightness;
	brightness=clamp(brightness,0.5,1.0);	
	vec3 R = reflect(lightvector, vec3(normal.x,normal.y,-normal.z));
	vec3 S = normalize( modelvertex.xyz - cameraposition );
	S.x *= -1.0;
	//S.z *= -1.0;


	//float gloss = (0.5 + 0.5 * texture2D(texture9,modelvertex.xz/10.0).x) * 50.0;
	gloss=80.0;
	float specular = 0.25 * pow(clamp(dot(R, S ),0.0,1.0),20.0 );
	specular=min(specular,1.0);	
	
	normal = T * normal.x + B * normal.y + N * normal.z;
	
	vec3 rnormal = cross( normal, N);
	//normal = cross( normal, reflectdir);
	
	//=================================
	// Reflection
	//=================================
	
	vec4 color = texture2DProj(texture0, projectspace + amplitude * vec4(rnormal.x,rnormal.y,rnormal.z,0.0) );

	color = fragcolor*0.5+color*0.5;

	//color *= texture2D(texture2,vec2(texcoord.x*4.0+0.5,texcoord.y*4.0) );
	
	// Depth alpha
	float depthfade=1.0;
	vec2 screencoord = vec2( gl_FragCoord.x/buffersize.x, gl_FragCoord.y/buffersize.y );	
	float depth = texture2D( texture2, screencoord ).x;
	float linearbackgrounddepth = DepthToZPosition( depth );
	float lineardepth = DepthToZPosition( gl_FragCoord.z );	
	depthfade = clamp(-(lineardepth - linearbackgrounddepth)/ waterdepth, 0.0, 1.0 );
	//depthfade = clamp(-(lineardepth - linearbackgrounddepth)/waterdepth, 0.0, 1.0 );
	
	//=================================
	// Refraction / wobble
	//=================================

	normal = normalize(bumpcolor.xyz - 0.5);
	normal = T * normal.x + B * normal.y + N * normal.z;
	rnormal = cross( normal, N);
	
	//vec4 adjustedprojspace = projectspace;
	//adjustedprojspace.y = adjustedprojspace.y*0.75+0.125;
	
	float refractionamplitude = clamp(linearbackgrounddepth - lineardepth,0.0,1.0) * wobbleamplitude;	
	vec4 refractionvector = refractionspace + refractionamplitude * vec4(rnormal.x,rnormal.y,rnormal.z,0.0);
	if (texture2DProj(texture2,refractionvector).x<gl_FragCoord.z) {
		refractionvector = refractionspace;
	}
	vec4 refraction = texture2DProj(texture4, refractionvector );
	//refraction = refraction * surffade + refraction * (1.0 - surffade);

	//=================================
	//
	//=================================


	//=================================
	// surf
	//=================================
	
	float surffade;
	surffade = -(lineardepth - linearbackgrounddepth)/3.0; 
	surffade = 1.0 - clamp(surffade ,0.0, 1.0 );
	vec4 surf;
	
	if (surffade>0.0) {	
		surf = texture2D(texture3,vec2(modelvertex.x/5.0 + bumpcolor.x * 0.025 + apptime / 30000.0,0.5+modelvertex.z * 0.025 + modelvertex.z/5.0 + apptime / 30000.0));
		surf += texture2D(texture3,vec2(0.5+modelvertex.z/5.0 + bumpcolor.y * 0.025 - apptime / 30000.0,modelvertex.x * 0.025 + modelvertex.x/5.0 - apptime / 30000.0));

//		surf = texture2D(texture3,vec2(texcoord.x/2.0 + bumpcolor.x * 0.05 + apptime / 30000.0,bumpcolor.y * 0.05 + texcoord.y/2.0 + apptime / 30000.0));
//		surf += texture2D(texture3,vec2(texcoord.y/2.0 + bumpcolor.y * 0.05 - apptime / 30000.0,bumpcolor.x * 0.05 + texcoord.x/2.0 - apptime / 30000.0));
		surf=max(vec4(1.0),surf*2.0);
	}

	//=================================
	//
	//=================================
	
	//Mix in green color for shallow water
	float depthmix;
	depthmix=-(lineardepth-linearbackgrounddepth+10.0)/20.0; 
	depthmix=1.0-clamp(depthmix,0.0,1.0 );
	color=color * fragcolor * (1.0 - depthmix) + color * vec4(0.35,0.65,0.45,1) * depthmix;
	
	//=================================
	// Wave tips
	//=================================
	
	//waterheight =  texture2D(texture6,modelvertex.xz/10.0 - vec2(0.0,apptime/100000.0)).x;
	//waterheight += texture2D(texture6,modelvertex.xz/10.0 + vec2(0.5,apptime/100000.0)).x;
	//waterheight /= 2.0;
	//
	//if (waterheight>0.5) {
	//	float tipmix = clamp((waterheight-0.5)/0.5 * 4.0,0.0,1.0);
	//	color = color * (1.0-tipmix) + tipmix * vec4(1.0);
	//}
	
	//=================================
	//
	//=================================
	

	//=================================
	// Color
	//=================================
	
	//float waterdepth = linearbackgrounddepth-lineardepth;
	//color=texture2D(texture8,vec2(waterdepth/20.0,waterdepth/50.0));
	
	//=================================
	//	
	//=================================
	
	float foam;
	
	#ifndef LOWQUALITYWATER
	
		float height;
		oceancoord.x = modelvertex.x * MajorWaveScale;
		oceancoord.y = modelvertex.z * MajorWaveScale - appseconds * MajorWaveSpeed;
		height = texture2D(texture8,oceancoord).x;
	
		oceancoord.x = (-COS22*modelvertex.x+SIN22*modelvertex.z)*MajorWaveScale;
		oceancoord.y = (-SIN22*modelvertex.x-COS22*modelvertex.z)*MajorWaveScale - (-appseconds*MajorWaveSpeed);
		height += texture2D(texture8,oceancoord).x;
	
		oceancoord.x = modelvertex.x * MinorWaveScale;
		oceancoord.y = modelvertex.z * MinorWaveScale - appseconds * MinorWaveSpeed;
		height += 0.5*texture2D(texture8,oceancoord).x;
		
		oceancoord.x = (-COS22*modelvertex.x+SIN22*modelvertex.z)*MinorWaveScale;
		oceancoord.y = (-SIN22*modelvertex.x-COS22*modelvertex.z)*MinorWaveScale - (-appseconds*MinorWaveSpeed);
		height += 0.5*texture2D(texture8,oceancoord).x;
		
		height /= 3.0;

	//	foam = surfacedetail * max(0.0,height-0.5)*8.0;

	#endif
	
	

	//foam += texture2D(texture7,modelvertex.xz/2.0+vec2(0.0,-appseconds*0.1)).x*max(0.0,height-0.5)*4.0 * 0.5;
	//foam += texture2D(texture7,modelvertex.xz/2.0+vec2(0.5,appseconds*0.1)).x*max(0.0,height-0.5)*4.0 * 0.5;

	//specular = specular*surfacedetail;
	

	vec4 oceancolor = (foam+specular + vec4(0.0,fragcolor.z*surfacedetail*0.5,fragcolor.y*surfacedetail*0.5,0.0) + brightness * ( color + surf * surffade )) * depthfade;
	oceancolor=clamp(oceancolor,0.0,1.0);

	oceancolor = oceancolor + (1.0 - depthfade) * refraction;

	//Now add distance fog
	float fogeffect = clamp( 1.0 - (fogrange.y - lineardepth) / (fogrange.y - fogrange.x) , 0.0, 1.0 ) * fogcolor.w;
	
	gl_FragData[0] = oceancolor * (1.0 - fogeffect) + fogcolor * fogeffect;
	gl_FragData[1]=vec4(0.0,0.0,0.0,1.0);
	gl_FragData[2]=vec4(0.0,0.0,0.0,1.0);
	
}
