Include "MeshMatrix.vert"

uniform float aspect;

//#ifdef LW_SM4
uniform sampler2D texture6;//heightmap
//#endif

uniform float apptime;

varying vec4 projectspace;
varying vec2 texcoord;
varying vec3 T,B,N;
varying vec4 modelvertex;
uniform vec2 camerarange;
uniform float camerazoom;
varying vec4 refractionspace;
varying vec4 fragcolor;
//varying float oceanheight;

uniform vec3 cameraposition;

vec4 texture2D4(in sampler2D texture, in vec2 texcoord ) {

	float width=1024.0;
	float height=1024.0;

	float alpha = fract(texcoord.x * width - 0.5);
	float beta = fract(texcoord.y * height - 0.5);
	
	vec2 pixelsize = vec2(0.5/width,0.5/height);
	
	vec2 st_00 = vec2(texcoord.x-pixelsize.x,texcoord.y-pixelsize.y);
	vec2 st_10 = vec2(texcoord.x+pixelsize.x,texcoord.y-pixelsize.y);
	vec2 st_01 = vec2(texcoord.x-pixelsize.x,texcoord.y+pixelsize.y);
	vec2 st_11 = vec2(texcoord.x+pixelsize.x,texcoord.y+pixelsize.y);
	
	return ((1.0 - alpha)*(1.0 - beta) * texture2D(texture, st_00)
		+ alpha * (1.0 - beta) * texture2D(texture, st_10)
		+ (1.0 - alpha)*beta * texture2D(texture, st_01)
		+ alpha * beta * texture2D(texture, st_11));
}

#define COS22 0.924
#define SIN22 0.383

uniform float MajorWaveSpeed = 0.01;
uniform float MajorWaveScale = 0.005;
uniform float MinorWaveSpeed = 0.01;
uniform float MinorWaveScale = 0.005;
uniform float MajorWaveHeight = 0.0;

varying vec4 modelvertexcrest;

void main(){
		
	Include "GetMatrix.vert"

	fragcolor = gl_Color * color;

	float appseconds = apptime/1000.0;
	
	//mat3 nmat = mat3(mat[0].xyz,mat[1].xyz,mat[2].xyz);
	
	modelvertex = mat * gl_Vertex;
	
	float waveheight = 0.0;

	float wavecrest=0.0;

	//appseconds=50.0;
	//oceanheight=0.0;//
	//AMD cards can't do texture fetch correctly
	//#ifdef LW_SM4
		vec2 texcoord_;
		texcoord_.x = modelvertex.x * MajorWaveScale;
		texcoord_.y = modelvertex.z * MajorWaveScale - appseconds * MajorWaveSpeed;
		waveheight = texture2D(texture6,texcoord_).x;
		texcoord_.x = (-COS22*modelvertex.x+SIN22*modelvertex.z)*MajorWaveScale;
		texcoord_.y = (-SIN22*modelvertex.x-COS22*modelvertex.z)*MajorWaveScale - (-appseconds*MajorWaveSpeed);
		waveheight += texture2D(texture6,texcoord_).x;
		waveheight /= 2.0;
		float vertdist=length( modelvertex.xz-cameraposition.xz );
		float waveamplitude = min(1.0,4.0*(1.0-clamp((vertdist/100.0),0.0,1.0)));
		waveheight -= 0.5;
		//oceanheight=waveheight;//
		waveheight *= waveamplitude * MajorWaveHeight;
		modelvertex.y += waveheight;
		//if (oceanheight>0.0) {
		//	wavecrest += oceanheight/0.5 * 4.0;
		//}
	//#endif
	
	//modelvertex.z += wavecrest;
	projectspace = gl_ModelViewProjectionMatrix * modelvertex;//(modelvertex+vec4(0.0,0.0,wavecrest,0.0) ); 
	
	gl_Position = projectspace;
	
	//modelvertex.z += wavecrest*0.5;
	
	modelvertexcrest = modelvertex;
	//modelvertexcrest.z += wavecrest;
	
	texcoord = modelvertex.xz;

	refractionspace = projectspace;
	projectspace.xy = (projectspace.xy / camerazoom + vec2(projectspace.w)) * 0.5;
	refractionspace.xy = (refractionspace.xy + vec2(refractionspace.w)) * 0.5; 

	//texcoord = ( gl_Vertex.xz  ) * camerarange.y * 3.0 + mat[3].xz;
	
	T = gl_NormalMatrix * vec3(1,0,0);
	B = gl_NormalMatrix * vec3(0,0,1);
	N = gl_NormalMatrix * vec3(0,1,0);
	
	#ifdef __GLSL_CG_DATA_TYPES
		gl_ClipVertex = modelvertex;//gl_ModelViewMatrix * mat * gl_Vertex;
	#endif

}
