#extension GL_ARB_draw_buffers : enable

#define waveamplitude 0.1
#define wavesize 64.0
#define wavespeed 0.00001
#define visibility 5.0

uniform sampler2D texture0;// reflection normal map
uniform sampler2D texture1;// refraction normal map
uniform sampler2D texture2;// reflection color buffer
uniform sampler2D texture3;// refraction color buffer
uniform sampler2D texture4;// refraction depth buffer
uniform sampler2D texture5;// reflection depth buffer
uniform sampler2D texture6;// second texture frame
uniform sampler2D texture7;// caustics map
uniform sampler2D texture8;// caustics map frame2
uniform sampler2D texture9;// caustics map frame2
//uniform samplerCube texture9;

uniform float frameinterpolation;
uniform float contrast;
uniform float saturation;
uniform float brightness;
uniform vec2 camerarange;
uniform float camerazoom;
uniform vec2 buffersize;
uniform float apptime;
uniform float softness;
uniform vec3 cameraposition;
uniform float wavescale=1.0;
uniform vec2 fogrange;
uniform vec4 fogcolor;

uniform vec3 sunposition = vec3(1000,1000,1000);
uniform vec4 suncolor = vec4(1.0);

uniform vec4 causticsmappingaxes[2];

varying vec2 texcoord;
varying vec4 modelvertex;
varying vec4 fragcolor;

Include "abstract::DepthToZPosition.frag"


void main() {
	
	vec4 reflection;
	vec4 reflectionvector;
	vec4 refraction;
	vec4 refractionvector;
	vec3 normal;
	float refractiondepth;
	float alpha=fragcolor.w;
	float fogeffect;
	float reflectiondepth;
	float surfacefogeffect;
	vec4 color;
	
	//=================================
	//Depth
	//=================================
	
	refractiondepth=DepthToZPosition(texture2D(texture4,gl_FragCoord.xy/buffersize).x);
	float fragDepth = DepthToZPosition(gl_FragCoord.z);
	float depthdifference=refractiondepth-fragDepth;
	float lineardepth=fragDepth;
	alpha *= clamp(depthdifference/softness,0.0,1.0);
	float depthfade = clamp(depthdifference/softness,0.0,1.0); // used for refraction amplitude and color
	
	//Calculate fog strength at water surface
	surfacefogeffect = clamp( 1.0 - (fogrange.y - lineardepth) / (fogrange.y - fogrange.x) , 0.0, 1.0 ) * fogcolor.w;



	
	
	//=================================
	//Reflection
	//=================================
	
	//Get surface normal with four texture lookups
	//normal= texture2D(texture0,modelvertex.xz/wavesize+0.0*wavespeed).xyz * 2.0 - 1.0;
	//normal+=texture2D(texture0,modelvertex.xz/wavesize-apptime*wavespeed+0.5).xyz * 2.0 - 1.0;
	//normal+=texture2D(texture0,modelvertex.zx/wavesize+apptime*wavespeed).xyz * 2.0 - 1.0;
	//normal+=texture2D(texture0,modelvertex.zx/wavesize-apptime*wavespeed+0.5).xyz * 2.0 - 1.0;
	//normal /= 4.0;

	//normal = 0.25 * ( ((1.0-frameinterpolation)*texture2D(texture0,modelvertex.xz*0.01)  + frameinterpolation*texture2D(texture6,modelvertex.xz*0.01)).xyz  * 2.0 - 1.0);
	normal = (((1.0-frameinterpolation)*texture2D(texture0,modelvertex.zx*0.1+0.135) + frameinterpolation*texture2D(texture6,modelvertex.zx*0.1+0.135)).xyz * 2.0 - 1.0);
	normal = normalize(normal);
	
	vec3 onormal = normal;
	
	normal.x *= wavescale;
	normal.y *= wavescale;
	normal = normalize(normal);	

	reflectionvector = vec4( gl_FragCoord.x/buffersize.x, gl_FragCoord.y/buffersize.y, gl_FragCoord.z, 1.0 );
	reflectionvector += waveamplitude * vec4(normal,0.0);
	reflection=texture2DProj(texture2,reflectionvector);
	
	reflectiondepth=texture2DProj(texture5,reflectionvector).x;
	float z=DepthToZPosition(reflectiondepth);
	fogeffect = clamp( 1.0 - (fogrange.y - z) / (fogrange.y - fogrange.x) , 0.0, 1.0 ) * fogcolor.w;
	fogeffect*=(1.0 - int(reflectiondepth));
	
	//Subtract surface fog effect so fog strength matches the reflected world
	fogeffect -= surfacefogeffect;
	fogeffect = max(fogeffect,0.0);
	reflection = reflection * (1.0-fogeffect) + fogcolor * fogeffect;
	reflection *= fragcolor;
	
	
	//=================================
	//Lighting
	//=================================

	float brightness_=1.0;
	vec4 specular = vec4(0);
	
	//#ifdef LW_WATERLIGHTING
	// Calculate lighting	
	vec3 lightvector = normalize(sunposition-modelvertex.xyz);
	//normal=onormal;
	brightness_ = 1.0 * max(0.0,dot(normal.xzy,-lightvector));	
	brightness_ = brightness_ * 0.125 + 0.875;
	//brightness_=clamp(brightness_,0.5,1.0);	
	//normal.x*=4.0;
	//normal.y*=4.0;
	//normal=normalize(normal);

	
	if (suncolor.x+suncolor.y+suncolor.z>0.0) {
		vec3 R = reflect(lightvector, vec3(normal.x,normal.y,-normal.z));
		vec3 S = normalize( modelvertex.xyz - cameraposition );
		S.x *= -1.0;
		vec3 surfacenormal = normal;
		specular = suncolor*pow(clamp(dot(R, S ),0.0,1.0),200.0 );
	}

	//#endif
	
	//=================================
	//Refraction
	//=================================
	
	//normal= texture2D(texture1,modelvertex.xz*0.5+apptime*0.00001).xyz * 2.0 - 1.0;
	//normal+=texture2D(texture1,modelvertex.xz*0.5-apptime*0.00001+0.5).xyz * 2.0 - 1.0;
	//normal /= 2.0;
	normal  = ((1.0-frameinterpolation)*texture2D(texture0,modelvertex.xz*4.0)  + frameinterpolation*texture2D(texture6,modelvertex.xz*4.0)).xyz  * 2.0 - 1.0;

	refractionvector = vec4( gl_FragCoord.x/buffersize.x, gl_FragCoord.y/buffersize.y, gl_FragCoord.z, 1.0 );
	vec4 refractionvector2 = refractionvector + depthfade * 0.025 * vec4(normal,0.0);		
	if (gl_FragCoord.z<texture2DProj(texture4,refractionvector2).x) {
		refractionvector=refractionvector2;
	}
	refraction=texture2DProj(texture3,refractionvector);	
	
	//Break up the specular reflection
	//specular *= texture2D(texture9,modelvertex.xz/8.0).x;
	
	//=================================
	//Mix colors
	//=================================
	
	color = (specular + reflection) * alpha + ((refraction) * (1.0-alpha));
	vec4 outputcolor = color * (1.0-surfacefogeffect) + surfacefogeffect*fogcolor;
	
	//Brightness
	outputcolor.x *= brightness;
	outputcolor.y *= brightness;
	outputcolor.z *= brightness;
	
	//Contrast
	outputcolor.x = (outputcolor.x - 0.5) * contrast + 0.5;
	outputcolor.y = (outputcolor.y - 0.5) * contrast + 0.5;
	outputcolor.z = (outputcolor.z - 0.5) * contrast + 0.5;
	
	//Saturation
	float grayscale = outputcolor.r * 0.3 + outputcolor.g * 0.59 + outputcolor.b * 0.11;
	float satgray = clamp(grayscale*(1.0-saturation),0.0,1.0);
	outputcolor.x=satgray+saturation*outputcolor.x;
	outputcolor.y=satgray+saturation*outputcolor.y;
	outputcolor.z=satgray+saturation*outputcolor.z;
	
	gl_FragData[0] = outputcolor;
}
