
uniform vec2 particleoffset[4];
uniform float rotationspeed;
uniform float particlecount;
uniform float particlecycle;
uniform float particletick;
uniform vec2 particleradius;
uniform vec3 cameraposition;
uniform mat4 cameramatrix;
uniform float rotationoffset;
uniform float apptime;

varying vec3 vertexposition_;
varying vec2 texcoord0;
varying vec2 texcoord1;
varying vec3 particleposition;
varying float particlesize;
varying vec3 N;

varying vec4 fake_FragCoord1;
varying vec3 vertexposition;


mat4 matFromDirection( in vec3 dir ) {
	vec3 i;
	vec3 j;
	vec3 k;
	j=vec3(0.0,1.0,0.0);
	k=normalize(dir);
	i=normalize(cross(j,k));
	mat4 mat = mat4(0.0);
	mat[0]=vec4(i.x,i.y,i.z,0.0);
	mat[1]=vec4(j.x,j.y,j.z,0.0);
	mat[2]=vec4(k.x,k.y,k.z,0.0);
	mat[3].w=1.0;
	return mat;
}

void main(void) {

	float cycle;
	float index;
	
	vertexposition=gl_Vertex.xyz;

	if (gl_Vertex.w==-1.0) {;
		
		// Particle is hidden
		gl_Position = vec4(0.0,0.0,0.0,0.0);
		gl_FrontColor=vec4(1.0,1.0,1.0,0.0);
		
		}
	else
		{
		vec4 outcolor = gl_Color;

		//gl_FrontColor=gl_Color;
		
		index = floor( gl_Vertex.w / 4.0 );
		
		cycle = mod(particlecycle + (index * particletick)/(particletick * float(particlecount)),1.0);	
		cycle=clamp(cycle,0.0,1.0);		

		// Scale
		float radius = particleradius.x + ( clamp(cycle,0.0,1.0) * ( particleradius.y - particleradius.x ) );
		
		// Fade in
		if (cycle<0.05) {
			outcolor.a*=cycle/0.05;
		}
		
		// Fade out
		#ifdef LE_LINEARPARTICLEFADE
			outcolor.a*=clamp(1.0-cycle,0.0,1.0);
		#else
			if (cycle>0.75) {
				outcolor.a*=(1.0-cycle)/0.25;
			}
		#endif

		vec4 v = gl_ModelViewMatrix * vec4( gl_Vertex.xyz, 1.0 );
		vec3 ov = ( cameramatrix * v ).xyz;
		
		particleposition = (gl_ProjectionMatrix * v).xyz;
		vec2 offset = particleoffset[ int(mod( gl_Vertex.w, 4.0 )) ];// * radius;
		
		// Rotation
		float seed = ( ( index * 169.0 + apptime ) / 1000.0 ) * 3.14159265 * 2.0 * rotationspeed;
		if ( (floor( index/2.0 ) * 2.0) == index ) {
			seed *= -1.0;
		}

		float angle = rotationoffset + seed + index * 10.0 * rotationspeed;

		float sa = sin( angle );
		float ca = cos( angle );
		
		v.x += sa * offset.y * radius - ca * offset.x * radius;
		v.y += -ca * offset.y * radius - sa * offset.x * radius;
		
		texcoord0 = offset * 0.5 + 0.5;
		texcoord1.x = sa * offset.y - ca * offset.x;
		texcoord1.y = -ca * offset.y - sa * offset.x;
		texcoord1 = texcoord1 * 0.5 + 0.5;
		
		//texcoord0.x +=  sa * 0.5  - ca * 0.5 ;
		//texcoord0.y += -ca * 0.5  - sa * 0.5 ;
		
		vertexposition_ = ( cameramatrix * v ).xyz;
		N = normalize(vertexposition_ - ov);
		N = normalize(gl_NormalMatrix * N);
		
		vec4 vertpos = gl_ProjectionMatrix * v;
		gl_Position = vertpos;
		fake_FragCoord1=vertpos;
		
		#ifdef __GLSL_CG_DATA_TYPES
			gl_ClipVertex = v;
		#endif
		
		gl_FrontColor = outcolor;
		//gl_FrontColor.a=0.0;
		
		particlesize = radius;
		
		}
	}
