#extension GL_ARB_draw_buffers : enable

#define neardistancefade 0.1

uniform sampler2D texture0;
uniform sampler2D texture1;
uniform sampler2D texture12;//texture color map
uniform vec2 camerarange;
uniform float terrainsize;
uniform vec3 terrainscale;
uniform vec2 buffersize;

varying vec2 texcoord0;
varying float particlesize;
varying vec4 fake_FragCoord1;// not used, declared only because it exists in vertex shader
varying vec3 vertexposition;

float DepthToZPosition(in float depth) {
	return camerarange.x / (camerarange.y - depth * (camerarange.y - camerarange.x)) * camerarange.y;
}

void main(void) {
	vec4 color = gl_Color * texture2D( texture0, texcoord0 );

	vec2 screencoord = gl_FragCoord.xy/buffersize;
	float depth = texture2D( texture1, screencoord ).x;
	float linearbackgrounddepth = DepthToZPosition( depth );
	float lineardepth = DepthToZPosition( gl_FragCoord.z );
	float nearfade = clamp( lineardepth - neardistancefade, 0.0, 1.0 );	
	float depthfade = clamp(-(lineardepth - linearbackgrounddepth) / particlesize, 0.0, 1.0 );
	color.w *= depthfade * nearfade;
	
	if (color.w<0.003) {
		discard;
	}
	
	vec2 terraincoord = vec2(vertexposition.x,-vertexposition.z) / terrainsize + 0.5;
	float terrainresolution = terrainsize / terrainscale.x;
	terraincoord += 0.5 / terrainresolution;
	vec4 terraincolor = texture2D(texture12,terraincoord);
	color = vec4( color.xyz * terraincolor.xyz,color.w);
	
	gl_FragData[0] = color;
}
