uniform sampler2D texture0; //color
uniform sampler2D texture1; //depth
uniform sampler2D texture2; //normal
uniform sampler2D texture10; //noise texture for effects
uniform vec2 buffersize;
uniform vec2 camerarange;
uniform float aosizereduce;
uniform float camerazoom;
uniform float bufferaspect;
uniform float apptime;
uniform mat4 cameramat4;

include "depthtozposition.frag"

#define aointensity 4.0		// between 1 and 5 is good
#define aointensitydiv 2.5
#define aofalloff .35
#define aosamples 8			// dont change
#define aopass 4			// 4 * aopass = samples
#define aocut 0.6
#define aosize 0.5
#define aoscreenlocked 0		// enable this for screen locked AO

float readZPosition(in vec2 texcoord) {
	return DepthToZPosition( texture2D( texture1, texcoord + 0.25 / buffersize ).x );
}

mat3 vec3tomat3( in vec3 z ) {
	mat3 mat;
	mat[2]=z;
	vec3 v=vec3(z.z,z.x,-z.y);//make a random vector that isn't the same as vector z
	mat[0]=cross(z,v);//cross product is the x axis
	mat[1]=cross(mat[0],z);//cross product is the y axis
	return mat;
}

float compareDepths( in float depth1, in float depth2, in float m ) {
	//#define mindiff 0.1
	//#define middiff 0.6
	//#define maxdiff 1.6
	//#define enddiff 2.1
	float mindiff=0.05*m;
	float middiff=0.25*m;
	float maxdiff=1.30*m;
	float enddiff=1.50*m;
	
	float diff = (depth1-depth2);
	if (diff<mindiff) {
		return 1.0;
	}
	if (diff<middiff) {
		diff -= mindiff;
		return 1.0-diff/(middiff-mindiff);
	}
	if (diff<maxdiff) {
		return 0.0;
	}
	if (diff<enddiff) {
		diff -= maxdiff;
		return diff/(enddiff-maxdiff);
	}
	return 1.0;	
}

// Calculates the screen-space position of a pixel
// Make sure the depth lookup is using the right texture!!!
vec3 GetPixelPosition( in vec2 coord ) {
	vec3 pos;
	vec2 hbuffersize=buffersize/2.0;
	pos.z = DepthToZPosition(texture2D(texture1,coord).x);
	pos = vec3((((coord.x+0.5)/hbuffersize.x)-0.5) * 2.0,(((-coord.y+0.5)/hbuffersize.y)+0.5) * 2.0 / (hbuffersize.x/hbuffersize.y),pos.z);
	pos.x *= pos.z / camerazoom;
	pos.y *= -pos.z / camerazoom;
	return pos;
}

vec3 ScreenCoordToPosition( in vec2 coord ) {
	vec3 pos;
	vec2 hbuffersize=buffersize/2.0;
	pos.z = DepthToZPosition(texture2D(texture1,coord).x);
	pos.x = ((((coord.x+0.5)/hbuffersize.x)-0.5) * 2.0)*(pos.z / camerazoom);
	pos.y = ((((-coord.y+0.5)/hbuffersize.y)+0.5) * 2.0 / (hbuffersize.x/hbuffersize.y))*(-pos.z / camerazoom);
	return pos;
}

vec3 ScreenCoordToPosition2( in vec2 coord, in float z ) {
	vec3 pos;
	vec2 hbuffersize=buffersize/2.0;
	pos.z = z;//DepthToZPosition(texture2D(texture1,coord).x);
	pos.x = ((((coord.x+0.5)/hbuffersize.x)-0.5) * 2.0)*(pos.z / camerazoom);
	pos.y = ((((-coord.y+0.5)/hbuffersize.y)+0.5) * 2.0 / (hbuffersize.x/hbuffersize.y))*(-pos.z / camerazoom);
	return pos;
}

//Converts a screen position to texture coordinate
vec2 ScreenPositionToCoord( in vec3 pos ) {
	vec2 coord;
	vec2 hbuffersize=buffersize/2.0;
	pos.x /= (pos.z / camerazoom);
	coord.x = (pos.x / 2.0 + 0.5);
	
	pos.y /= (-pos.z / camerazoom) / (hbuffersize.x/hbuffersize.y);
	coord.y = -(pos.y / 2.0 - 0.5);
	
	return coord;// + 0.5/(buffersize/2.0);
}

vec4 PlaneFromPointNormal( in vec3 point, in vec3 normal ) {
	vec4 plane;
	plane.x = normal.x;
	plane.y = normal.y;
	plane.z = normal.z;
	plane.w = -dot(point,normal);
	return plane;
}

vec3 GetPixelNormal( in vec2 coord ) {
	return texture2D( texture2, coord ).xyz * 2.0 - 1.0;
}

vec4 GetPixelPlane( in vec2 coord ) {
	vec3 point = GetPixelPosition( coord );
	vec3 normal = GetPixelNormal( coord );
	return PlaneFromPointNormal( point, normal);
}

float PointPlaneDistance( in vec4 plane, in vec3 point ) {
	return plane.x*point.x+plane.y*point.y+plane.z*point.z+plane.w;
}

float rand(vec2 co) {
        return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main( void ) {
	float ao=0.0;
	float dn;
	float zd;
	float newdepth;
	float angletest;
	float lineardepth;
	float depth;
	vec3 newnormal;
	vec3 newdiffuse;
	vec3 normal;
	vec2 texcoord2;
	vec2 texcoord = gl_FragCoord.xy/buffersize + 0.5/(buffersize*0.5);
	depth=texture2D( texture1, texcoord ).x;
	vec3 screencoord;
	vec4 outcolor;
	
	if (depth<1.0) {
		
		lineardepth = DepthToZPosition(depth);
		normal = texture2D( texture2, texcoord ).rgb * 2.0 - 1.0;
		normal=normalize(normal);
		normal.z=-normal.z;
		
		vec3 p0=ScreenCoordToPosition2(vec2(0.0,0.5),lineardepth);
		vec3 p1=ScreenCoordToPosition2(vec2(1.0,0.5),lineardepth);
		float dist = abs(p1.x-p0.x);
		
		screencoord = vec3((((gl_FragCoord.x+0.5)/buffersize.x)-0.5) * 2.0,(((-gl_FragCoord.y+0.5)/buffersize.y)+0.5) * 2.0 / (buffersize.x/buffersize.y),lineardepth);
		screencoord.x *= screencoord.z / camerazoom;
		screencoord.y *= -screencoord.z / camerazoom;
		
		vec3 newpoint;
		vec2 coord;
		vec3 raynormal;
		vec3 offsetvector;
		float diff;		
		vec2 randcoord;		
		float randsum = cameramat4[0][0]+cameramat4[0][1]+cameramat4[0][2];
		randsum+=cameramat4[1][0]+cameramat4[1][1]+cameramat4[1][2];
		randsum+=cameramat4[2][0]+cameramat4[2][1]+cameramat4[2][2];
		randsum+=cameramat4[3][0]+cameramat4[3][1]+cameramat4[3][2];
		
		#define raycasts 4//Increase for better quality
		#define raysegments 4//Increase for better quality
		#define aoscatter 1.0
		float raylength = 0.25;//*dist*1.0*50.0;
		float cdm = 1.0;// * dist * 50.0;
		
		ao=0.0;
		
		vec4 gicolor;
		float gisamples;
		mat3 mat=vec3tomat3(normal);
		float a;
		float wheredepthshouldbe;
		
		float mix=(1.0-(clamp(lineardepth-50.0,0.0,50.0)/50.0));
		
		//Get a random number
		a=rand( randsum+texcoord);// + float(53.0*m*raysegments + i*13.0) );
		
		vec4 gi=vec4(0);
		float aof;
		float gisum;
		
		if (mix>0.0) {

			for ( int i=0;i<(raycasts);i++ ) {
				for ( int m=0;m<(raysegments);m++ ) {
					
					//float scatteroffset=rand( randsum+texcoord+i+m)*0.5+0.5;
					
					offsetvector.x=cos(a+float(i)/float(raycasts)*3.14*4.0)*aoscatter;
					offsetvector.y=sin(a+float(i)/float(raycasts)*3.14*4.0)*aoscatter;
					offsetvector.z=1.0;
					offsetvector=normalize(offsetvector);
					
					//Create the ray vector
					raynormal=mat*offsetvector;
					
					//Add the ray vector to the screen position
					newpoint = screencoord + raynormal * (raylength/raysegments) * float(m+1);
					wheredepthshouldbe=newpoint.z;
					
					//Turn the point back into a screen coord:
					coord = ScreenPositionToCoord( newpoint );
					
					//Look up the depth value at that rays position
					newdepth = readZPosition( coord );
					
					//If new depth is closer to camera darken ao value
					//rayao = max(rayao,compareDepths( lineardepth, newdepth ));
					//ao = min(ao,compareDepths( lineardepth, newdepth ));
					//aof = compareDepths( wheredepthshouldbe,newdepth, cdm );
					ao += compareDepths( wheredepthshouldbe,newdepth, cdm );
					
					//if (aof>0.0) {
					//	gisum = gisum+(1.0-aof);					
					//	gi += texture2D( texture0, coord ) * (1.0-aof);
					//}
					//ao+=compareDepths( lineardepth, newdepth );
				}
			}
		}
		
		gi /= gisum;
		
		ao /= (raycasts*raysegments);
		//ao = max(ao * ao,0.5);
		ao = ao * mix + (1.0-mix);	
		//if (gisum>0.0) {
		//	gi /= gisum;
		//}
		//else {
		//	gi=vec4(0);
		//}
		//if (texcoord.x>0.5) {
		//	ao=1.0;
		//}
		gl_FragColor=vec4(ao);
	}
	
	else {
		gl_FragColor=vec4(1.0);
	}
	
}
