uniform sampler2D texture0;//color
uniform sampler2D texture1;//depth
uniform sampler2D texture2;//normal
uniform sampler2D texture10;//noise

uniform vec2 camerarange;
uniform vec2 buffersize;

float DepthToLinearDepth(in float depth) {
	return (camerarange.x / (camerarange.y - depth * (camerarange.y - camerarange.x)) * camerarange.y)/(camerarange.y-camerarange.x);
}

vec3 readNormal(in vec2 coord)  
{  
     return normalize(texture2D(texture2, coord).xyz*2.0  - 1.0);  
}

vec3 posFromDepth(vec2 coord){
     float d = texture2D(texture1, coord.xy).r;
	 d=DepthToLinearDepth(d);
     vec3 tray = mat3x3(gl_ProjectionMatrixInverse)*vec3((coord.x-0.5)*2.0,(coord.y-0.5)*2.0,1.0);
     return tray*d;
}
    //Ambient Occlusion form factor:
    float aoFF(in vec3 ddiff,in vec3 cnorm, in float c1, in float c2, in vec2 coord){
          vec3 vv = normalize(ddiff);
          float rd = length(ddiff);
          return (1.0-clamp(dot(readNormal(coord.xy+vec2(c1,c2)),-vv),0.0,1.0)) *
           clamp(dot( cnorm,vv ),0.0,1.0)* 
                 (1.0 - 1.0/sqrt(1.0/(rd*rd) + 1.0));
    }
    //GI form factor:
    float giFF(in vec3 ddiff,in vec3 cnorm, in float c1, in float c2, in vec2 coord){
          vec3 vv = normalize(ddiff);
          float rd = length(ddiff);
          return 1.0*clamp(dot(readNormal(coord.xy+vec2(c1,c2)),-vv),0.0,1.0)*
                     clamp(dot( cnorm,vv ),0.0,1.0)/
                     (rd*rd+1.0);  
    }

void main()
{

	vec2 texcoord = gl_FragCoord.xy/buffersize;

    //read current normal,position and color.
    vec3 n = readNormal(texcoord.st);
    vec3 p = posFromDepth(texcoord.st);
    vec3 col = texture2D(texture0, texcoord.xy).rgb;

    //randomization texture
    vec2 fres = vec2(800.0/128.0*5,600.0/128.0*5);
    vec3 random = texture2D(texture10, texcoord.st*fres.xy).xyz;
    random = random*2.0-vec3(1.0);
	
    //initialize variables:
    float ao = 0.0;
    vec3 gi = vec3(0.0,0.0,0.0);
    float incx = 1.0/800.0*0.1;
    float incy = 1.0/600.0*0.1;
    float pw = incx;
    float ph = incy;
    float cdepth = DepthToLinearDepth(texture2D(texture1, texcoord.xy).r);
	
    //3 rounds of 8 samples each. 
    for(float i=0.0; i<3.0; ++i) 
    {
       float npw = (pw+0.0007*random.x)/cdepth;
       float nph = (ph+0.0007*random.y)/cdepth;

       vec3 ddiff = posFromDepth(texcoord.st+vec2(npw,nph))-p;
       vec3 ddiff2 = posFromDepth(texcoord.st+vec2(npw,-nph))-p;
       vec3 ddiff3 = posFromDepth(texcoord.st+vec2(-npw,nph))-p;
       vec3 ddiff4 = posFromDepth(texcoord.st+vec2(-npw,-nph))-p;
       vec3 ddiff5 = posFromDepth(texcoord.st+vec2(0,nph))-p;
       vec3 ddiff6 = posFromDepth(texcoord.st+vec2(0,-nph))-p;
       vec3 ddiff7 = posFromDepth(texcoord.st+vec2(npw,0))-p;
       vec3 ddiff8 = posFromDepth(texcoord.st+vec2(-npw,0))-p;

       ao+=  aoFF(ddiff,n,npw,nph,texcoord);
       ao+=  aoFF(ddiff2,n,npw,-nph,texcoord);
       ao+=  aoFF(ddiff3,n,-npw,nph,texcoord);
       ao+=  aoFF(ddiff4,n,-npw,-nph,texcoord);
       ao+=  aoFF(ddiff5,n,0,nph,texcoord);
       ao+=  aoFF(ddiff6,n,0,-nph,texcoord);
       ao+=  aoFF(ddiff7,n,npw,0,texcoord);
       ao+=  aoFF(ddiff8,n,-npw,0,texcoord);

       gi+=  giFF(ddiff,n,npw,nph,texcoord)*texture2D(texture0, texcoord.xy+vec2(npw,nph)).rgb;
       gi+=  giFF(ddiff2,n,npw,-nph,texcoord)*texture2D(texture0, texcoord.xy+vec2(npw,-nph)).rgb;
       gi+=  giFF(ddiff3,n,-npw,nph,texcoord)*texture2D(texture0, texcoord.xy+vec2(-npw,nph)).rgb;
       gi+=  giFF(ddiff4,n,-npw,-nph,texcoord)*texture2D(texture0, texcoord.xy+vec2(-npw,-nph)).rgb;
       gi+=  giFF(ddiff5,n,0,nph,texcoord)*texture2D(texture0, texcoord.xy+vec2(0,nph)).rgb;
       gi+=  giFF(ddiff6,n,0,-nph,texcoord)*texture2D(texture0, texcoord.xy+vec2(0,-nph)).rgb;
       gi+=  giFF(ddiff7,n,npw,0,texcoord)*texture2D(texture0, texcoord.xy+vec2(npw,0)).rgb;
       gi+=  giFF(ddiff8,n,-npw,0,texcoord)*texture2D(texture0, texcoord.xy+vec2(-npw,0)).rgb;

       //increase sampling area:
       pw += incx;  
       ph += incy;    
    } 
    ao/=24.0;
    gi/=24.0;
	
    gl_FragColor = vec4(col-vec3(ao)+gi*5.0,1.0);
}




