#extension GL_ARB_draw_buffers : enable

//uniform sampler2D texture14;// normal map


uniform sampler2D texture0;// texture0
uniform sampler2D texture1;// bumpmap0
uniform sampler2D texture2;// texture1
uniform sampler2D texture3;// bumpmap1
uniform sampler2D texture4;// texture2
uniform sampler2D texture5;// bumpmap2
uniform sampler2D texture6;// texture3
uniform sampler2D texture7;// bumpmap3
uniform sampler2D texture8;// texture4
uniform sampler2D texture9;// bumpmap4
uniform sampler2D texture10;// basetexture
uniform sampler2D texture11;// tilemap
uniform sampler2D texture12;// colormap
uniform sampler2D texture13;// alphatexture
uniform sampler2D texture14;// normaltexture
uniform sampler2D texture15;//heightmap


uniform vec3 cameraposition;

uniform float layerblend[LW_TERRAINLAYERS];
uniform vec4 layercolor[ LW_TERRAINLAYERS ];
uniform vec4 layerconstraints[ LW_TERRAINLAYERS ];
uniform vec2 layerposition[ LW_TERRAINLAYERS ];
uniform float layerscale[ LW_TERRAINLAYERS ];
uniform vec2 layerrotation[ LW_TERRAINLAYERS ];
uniform float terrainaspect;
uniform vec3 terrainscale;
uniform float terrainresolution;
uniform float terrainsize;

varying vec3 vertexposition;
varying vec3 texcoord0;
varying vec2 texcoord1;
varying vec3 T,B;

vec4 terrainTexture2D(  in sampler2D tex, in float scale, in vec2 rotation ) {
	
	//----------------------------------
	//Slower version with rotation
	//----------------------------------
	vec2 coord;
	coord.x = ( texcoord0.x * rotation.x + texcoord0.z * rotation.y ) / scale - 0.5;
	coord.y = ( texcoord0.z * rotation.x - texcoord0.x * rotation.y ) / scale - 0.5;
	return texture2D( tex, coord );
	
	//----------------------------------
	//Faster version without rotation
	//----------------------------------
	//return texture2D( tex, texcoord0.xz / scale );
	
}

vec4 terrainTexture3D(  in sampler2D tex, in vec2 axes, in float scale, in vec2 rotation, in float height ) {

	//----------------------------------
	//Slower version with rotation
	//----------------------------------	
	return 	texture2D(tex,vec2( texcoord0.x * rotation.x + height * rotation.y, height * rotation.x - texcoord0.x * rotation.y) / scale ) * axes.y
	+ 	texture2D(tex,vec2( texcoord0.z * rotation.x + height * rotation.y, height * rotation.x - texcoord0.z * rotation.y) / scale ) * axes.x;
	
	//----------------------------------
	//Faster version without rotation
	//----------------------------------
	//return 	texture2D(tex,texcoord0.xy/scale) * axes.y
	//+ 	texture2D(tex,texcoord0.zy/scale) * axes.x;

}

void main(void) {
	
	//Discard test if tile map used - used for hiding tiles
	#ifdef LW_VISIBILITYMAP
		vec2 vmcoord;
		vmcoord = vertexposition.xz / terrainsize + 0.5;
		//if (texture2D(texture11,texcoord1).x<0.5) {


		//float pixelsize = 1.0 / terrainresolution;
		//vmcoord = vertexposition.xz / terrainsize - 0.5;
		//vmcoord += 0.5 * pixelsize;

		if (texture2D(texture11,vmcoord).x<0.5) {
			discard;
		}
	#endif
	
	float height;
	#ifdef LW_RENDERTERRAINCOLORMAP
		height = texture2D(texture15,vec2(texcoord1.x,1.0-texcoord1.y)).x*terrainscale.y;
	#else
		//No idea why this changes behavior:
		height = texcoord0.y;//vertexposition.y;
	#endif

	vec3 n0;
	vec3 n1;

	vec4 albedo;
	vec4 alpha = texture2D(texture13,vec2(texcoord1.x,1.0-texcoord1.y));
	vec4 bumpcolor;
	vec3 normal;
	vec3 worldNormal = ((texture2D(texture14,texcoord1).xyz - 0.5) * 2.0).xyz;
	
	float specular = 0.0;
	
	vec3 N = normalize( gl_NormalMatrix * worldNormal);
	vec3 bumpnormal = vec3(1.0,1.0,1.0);
	float shininess = 0.0;
	normal=N;
	vec2 coord;	
	vec2 mix;	
	
	float sum = abs(worldNormal.x) + abs(worldNormal.z);
	mix.x = abs(worldNormal.x) / sum;
	mix.y = abs(worldNormal.z) / sum;
	
	float slope = asin( worldNormal.y );
	slope = 90.0 - slope * 57.2957795;
	vec4 layerdiffuse;
	vec4 layerdiffuse1;
	
	int usebumpcolor;
	
	vec4 background;
	
	float bumpcontribs = 1.0;	
	
	//#ifndef LW_RENDERTERRAINCOLORMAP

	//if (length(cameraposition-vertexposition)>50.0) {
	//	
	//	albedo = texture2D(texture12,texcoord1);
	//	bumpnormal=N;
	//	
	//}
	
	//else {

	//#endif

	//=====================================
	// Layer constraints
	//=====================================
	for ( int i = 0; i < 4; i++ ) {
		alpha[i] *= clamp( ( slope - layerconstraints[i].x)/5.0,0.0,1.0);
		alpha[i] *= clamp( ( layerconstraints[i].y-slope)/5.0,0.0,1.0);
		alpha[i] *= clamp( ( height - layerconstraints[i].z), 0.0, 1.0 );
		alpha[i] *= clamp( ( layerconstraints[i].w - height ), 0.0, 1.0 );		
	}
	
	#ifdef LW_BASETEXTURE
		background=texture2D(texture12,texcoord1.xy);
		background.w=1.0;
	#else
		background=vec4(0.5);
	#endif
	
	// ====================================================================
	// Texture 0
	// ====================================================================
	#ifdef LW_LAYER0
		#ifdef LW_LAYER0_VERTICAL
			layerdiffuse = terrainTexture3D( LW_LAYER0, mix, layerscale[0], layerrotation[0], height );
		#else
			//layerdiffuse = texture2D( LW_LAYER0, vec2(1.0,-1.0) * texcoord1 );
			layerdiffuse = terrainTexture2D( LW_LAYER0, layerscale[0], layerrotation[0] );
		//	layerdiffuse = layerdiffuse*0.5+0.5*terrainTexture2D( LW_LAYER0, layerscale[0]*10.0, layerrotation[0] );
		#endif
		layerdiffuse = layerdiffuse * layerblend[0] + layerdiffuse * background * 2.0 * (1.0-layerblend[0]);
		albedo = layerdiffuse * layercolor[0];
	#else
		albedo = vec4(0.5);
	#endif
	// ====================================================================
	//
	// ====================================================================
	

	// ====================================================================
	// Bumpmap 0
	// ====================================================================
	#ifdef LW_BUMPMAP0
		#ifdef LW_LAYER0_VERTICAL
		//	layerdiffuse += terrainTexture3D( LW_BUMPMAP0, mix, layerscale[0], layerrotation[0], height )*0.5;
		//	layerdiffuse += terrainTexture3D( LW_BUMPMAP0, mix, layerscale[0]/10.0, layerrotation[0], height )*0.5;
			layerdiffuse = terrainTexture3D( LW_BUMPMAP0, mix, layerscale[0], layerrotation[0], height );
		#else
			layerdiffuse = terrainTexture2D( LW_BUMPMAP0, layerscale[0], layerrotation[0] );
		//	layerdiffuse = layerdiffuse*0.5+0.5*terrainTexture2D( LW_BUMPMAP0, layerscale[0]*10.0, layerrotation[0] );
		#endif
		bumpcolor = layerdiffuse;
		bumpcolor.w *= layercolor[0].w;
	#else
		bumpcolor = vec4(0.5,0.5,1.0,0.0);
	#endif
	// ====================================================================
	//
	// ====================================================================
	

	// ====================================================================
	// Texture 1
	// ====================================================================	
	#ifdef LW_LAYER1
		#ifdef LW_LAYER1_VERTICAL
			layerdiffuse = terrainTexture3D( LW_LAYER1, mix, layerscale[1], layerrotation[1], height );
		#else
			layerdiffuse = terrainTexture2D( LW_LAYER1, layerscale[1], layerrotation[1] );
		#endif
		layerdiffuse = layerdiffuse * layerblend[1] + layerdiffuse * background * 2.0 * (1.0-layerblend[1]);

		//	#ifdef LW_BLEND1
	//		layerdiffuse = layerdiffuse * background * 2.0;
	//	#endif
		albedo = (1.0 - alpha[0]) * albedo + (alpha[0] * layerdiffuse * layercolor[1]);
	#endif
	// ====================================================================
	//
	// ====================================================================
	
	
	// ====================================================================
	// Bumpmap 1
	// ====================================================================
	#ifdef LW_BUMPMAP1
		#ifdef LW_LAYER1_VERTICAL
			layerdiffuse = terrainTexture3D( LW_BUMPMAP1, mix, layerscale[1], layerrotation[1], height )*0.5;
			layerdiffuse += terrainTexture3D( LW_BUMPMAP1, mix, layerscale[1]/10.0, layerrotation[1], height )*0.5;
//			layerdiffuse = terrainTexture3D( LW_BUMPMAP1, mix, layerscale[1], layerrotation[1], height );
		#else
			layerdiffuse = terrainTexture2D( LW_BUMPMAP1, layerscale[1], layerrotation[1] );
		#endif
		layerdiffuse.w *= layercolor[1].w;
		bumpcolor = (1.0 - alpha[0]) * bumpcolor + (alpha[0] * layerdiffuse);
	#endif
	// ====================================================================
	//
	// ====================================================================


	// ====================================================================
	// Texture 2
	// ====================================================================	
	#ifdef LW_LAYER2
		#ifdef LW_LAYER2_VERTICAL
			layerdiffuse = terrainTexture3D( LW_LAYER2, mix, layerscale[2], layerrotation[2], height );
		#else
			layerdiffuse = terrainTexture2D( LW_LAYER2, layerscale[2], layerrotation[2] );
		#endif
		layerdiffuse = layerdiffuse * layerblend[2] + layerdiffuse * background * 2.0 * (1.0-layerblend[2]);
		//#ifdef LW_BLEND2
			//layerdiffuse = layerdiffuse * background * 2.0;
		//#endif
		albedo = (1.0 - alpha[1]) * albedo + (alpha[1] * layerdiffuse * layercolor[2]);
	#endif
	// ====================================================================
	//
	// ====================================================================
	
	
	// ====================================================================
	// Bumpmap 2
	// ====================================================================
	#ifdef LW_BUMPMAP2
		#ifdef LW_LAYER2_VERTICAL
			n0 = terrainTexture3D( LW_BUMPMAP2, mix, layerscale[2], layerrotation[2], height ).xyz * 2.0 - 1.0;
			n0 += terrainTexture3D( LW_BUMPMAP2, mix, layerscale[2]/10.0, layerrotation[2], height ).xyz * 2.0 - 1.0;
			layerdiffuse.xyz = n0 * 0.5 + 0.5;
		#else
			layerdiffuse = terrainTexture2D( LW_BUMPMAP2, layerscale[2], layerrotation[2] );
		#endif
		layerdiffuse.w *= layercolor[2].w;
		bumpcolor = (1.0 - alpha[1]) * bumpcolor + (alpha[1] * layerdiffuse);
		//bumpcolor.w *= layercolor[1].w;
	#endif
	// ====================================================================
	//
	// ====================================================================	
	

	// ====================================================================
	// Texture 3
	// ====================================================================	
	#ifdef LW_LAYER3
		#ifdef LW_LAYER3_VERTICAL
			layerdiffuse = terrainTexture3D( LW_LAYER3, mix, layerscale[3], layerrotation[3], height );
		#else
			layerdiffuse = terrainTexture2D( LW_LAYER3, layerscale[3], layerrotation[3] );
		#endif
		//#ifdef LW_BLEND3
		//	layerdiffuse = layerdiffuse * background * 2.0;
		//#endif
		layerdiffuse = layerdiffuse * layerblend[3] + layerdiffuse * background * 2.0 * (1.0-layerblend[3]);
		albedo = (1.0 - alpha[2]) * albedo + (alpha[2] * layerdiffuse * layercolor[3]);
	#endif
	// ====================================================================
	//
	// ====================================================================
	
	
	// ====================================================================
	// Bumpmap 3
	// ====================================================================
	#ifdef LW_BUMPMAP3
		#ifdef LW_LAYER3_VERTICAL
//			layerdiffuse = terrainTexture3D( LW_BUMPMAP3, mix, layerscale[3], layerrotation[3], height )*0.5;
//			layerdiffuse += terrainTexture3D( LW_BUMPMAP3, mix, layerscale[3]/10.0, layerrotation[3], height )*0.5;
			layerdiffuse = terrainTexture3D( LW_BUMPMAP3, mix, layerscale[3], layerrotation[3], height );
		#else
			layerdiffuse = terrainTexture2D( LW_BUMPMAP3, layerscale[3], layerrotation[3] );
		#endif
		layerdiffuse.w *= layercolor[3].w;
		bumpcolor = (1.0 - alpha[2]) * bumpcolor + (alpha[2] * layerdiffuse);
		//bumpcolor.w *= layercolor[2].w;
	#endif
	// ====================================================================
	//
	// ====================================================================	

	
	// ====================================================================
	// Texture 4
	// ====================================================================	
	#ifdef LW_LAYER4
		#ifdef LW_LAYER4_VERTICAL
			layerdiffuse = terrainTexture3D( LW_LAYER4, mix, layerscale[4], layerrotation[4], height );
		//	layerdiffuse = layerdiffuse*0.5+0.5*terrainTexture3D( LW_LAYER4, mix, layerscale[4]*10.0, layerrotation[4], height );
		#else
			layerdiffuse = terrainTexture2D( LW_LAYER4, layerscale[4], layerrotation[4] );
		#endif
		//#ifdef LW_BLEND4
		//	layerdiffuse = layerdiffuse * background * 2.0;
		//#endif
		layerdiffuse = layerdiffuse * layerblend[4] + layerdiffuse * background * 2.0 * (1.0-layerblend[4]);
		albedo = (1.0 - alpha[3]) * albedo + (alpha[3] * layerdiffuse * layercolor[4]);
	#endif
	// ====================================================================
	//
	// ====================================================================
	
	
	// ====================================================================
	// Bumpmap 4
	// ====================================================================
	#ifdef LW_BUMPMAP4
		#ifdef LW_LAYER4_VERTICAL
//			layerdiffuse += terrainTexture3D( LW_BUMPMAP4, mix, layerscale[4], layerrotation[4], height )*0.5;
//			layerdiffuse += terrainTexture3D( LW_BUMPMAP4, mix, layerscale[4]/10.0, layerrotation[4], height )*0.5;

			layerdiffuse = terrainTexture3D( LW_BUMPMAP4, mix, layerscale[4], layerrotation[4], height );
		//	layerdiffuse = layerdiffuse*0.5+0.5*terrainTexture3D( LW_BUMPMAP4, mix, layerscale[4]*10.0, layerrotation[4], height );
		#else
			layerdiffuse = terrainTexture2D( LW_BUMPMAP4, layerscale[4], layerrotation[4] );
		#endif
		layerdiffuse.w *= layercolor[4].w;
		bumpcolor = (1.0 - alpha[3]) * bumpcolor + (alpha[3] * layerdiffuse);
		bumpcolor.w *= layercolor[3].w;
	#endif
	// ====================================================================
	//
	// ====================================================================	
	
	specular=bumpcolor.w;
	bumpnormal = bumpcolor.xyz * 2.0 - 1.0;
	
	//Variation texture
	//albedo *= texture2D(texture12,texcoord0.xz/terrainscale.xz/1024.0)*2.0;
	
	normal = T * bumpnormal.x + B * bumpnormal.y + N * bumpnormal.z;
	normal=normalize(normal);
	
	gl_FragData[0] = vec4(albedo.xyz,specular);
	gl_FragData[1] = vec4(normal*0.5+0.5,specular);
	//specular=0;
	gl_FragData[2] = vec4(specular,0.5,0,1);
	gl_FragData[3] = vec4(0);
}