uniform vec3 cameraposition;
uniform vec2 billboardviewrange;

varying vec4 fragcolor;
varying vec2 texcoord0;
varying vec3 T,B,N;

int MajorAxis( in vec3 v ) {
	if (abs(v.z)>abs(v.x)) {
		if (abs(v.z)>abs(v.y)) {
			return int(3*sign(v.z));
		}
		else {
			return int(2*sign(v.y));
		}
	}
	else {
		if (abs(v.x)>abs(v.y)) {
			return int(1*sign(v.x));
		}
		else {
			return int(2*sign(v.y));
		}
	}
}

void main(void) {
	
	vec4 v=vec4(0,0,0,1);
	vec3 t,b,n;
	vec2 texcoordoffset;	
	int mainaxis;
	mat4 mat;
	
	//Reconstruct billboard 4x4 matrix
	mat[0][0]=gl_Normal.x;
	mat[0][1]=gl_Normal.y;
	mat[0][2]=gl_Normal.z;
	mat[0][3]=0.0;
	mat[1][0]=gl_MultiTexCoord2.x;
	mat[1][1]=gl_MultiTexCoord2.y;
	mat[1][2]=gl_MultiTexCoord2.z;
	mat[1][3]=0.0;
	mat[2][0]=gl_MultiTexCoord3.x;
	mat[2][1]=gl_MultiTexCoord3.y;
	mat[2][2]=gl_MultiTexCoord3.z;
	mat[2][3]=0.0;
	mat[3]=gl_Vertex;
	
	//Create inverse 3x3 matrix
	mat3 imat;
	imat[0][0]=mat[0][0];
	imat[0][1]=mat[1][0];
	imat[0][2]=mat[2][0];
	imat[1][0]=mat[0][1];
	imat[1][1]=mat[1][1];
	imat[1][2]=mat[2][1];
	imat[2][0]=mat[0][2];
	imat[2][1]=mat[1][2];
	imat[2][2]=mat[2][2];
	
	//Calculate normal matrix
	mat3 nmat = gl_NormalMatrix * mat3(mat[0].xyz,mat[1].xyz,mat[2].xyz);
	
	//Get vector between camera position and billboard center relative to billboard orientation
	vec3 dir = cameraposition - gl_Vertex.xyz;
	
	//Get vector length for later
	float l=length(dir);	

	//Get vector relative to object space
	dir=imat*dir;	
	
	//Get most influential axis
	mainaxis = MajorAxis( dir );
	
	//Adjust texture offset and billboard orientation based on major axis
	if (mainaxis==1) {
		texcoordoffset=vec2(0.25,0.0);
		v.z = gl_MultiTexCoord1.x;
		v.y = gl_MultiTexCoord1.y;
		t=vec3(0,0,1);
		b=vec3(0,1,0);
		n=vec3(1,0,0);
	}
	if (mainaxis==-1) {
		texcoordoffset=vec2(0.75,0.0);
		v.z =-gl_MultiTexCoord1.x;
		v.y = gl_MultiTexCoord1.y;
		t=vec3(0,0,-1);
		b=vec3(0,1,0);
		n=vec3(-1,0,0);
	}
	if (mainaxis==2) {
		texcoordoffset=vec2(0.0,0.5);
		v.x = gl_MultiTexCoord1.x;
		v.z = gl_MultiTexCoord1.y;
		t=vec3(1,0,0);
		b=vec3(0,0,1);
		n=vec3(0,1,0);
	}
	if (mainaxis==-2) {
		texcoordoffset=vec2(0.25,0.5);
		v.x = gl_MultiTexCoord1.x;
		v.z =-gl_MultiTexCoord1.y;
		t=vec3(1,0,0);
		b=vec3(0,0,-1);
		n=vec3(0,-1,0);
	}
	if (mainaxis==3) {
		texcoordoffset=vec2(0.5,0.0);
		v.x =-gl_MultiTexCoord1.x;
		v.y = gl_MultiTexCoord1.y;
		t=vec3(-1,0,0);
		b=vec3(0,1,0);
		n=vec3(0,0,1);
	}
	if (mainaxis==-3) {
		texcoordoffset=vec2(0.0,0.0);
		v.x = gl_MultiTexCoord1.x;
		v.y = gl_MultiTexCoord1.y;
		t=vec3(1,0,0);
		b=vec3(0,1,0);
		n=vec3(0,0,-1);
	}
	
	//Discard if out of range
	float discardrange = 0.5+0.5*abs(mat[3].x-int(mat[3].x));
	if (l<billboardviewrange.x) {
		v=vec4(0);
	}
	if (l>billboardviewrange.x+(billboardviewrange.y-billboardviewrange.x)*discardrange) {
		v=vec4(0);
	}

	v = mat*v;

	#ifdef __GLSL_CG_DATA_TYPES
		gl_ClipVertex = v;
	#endif
	
	gl_Position = gl_ModelViewProjectionMatrix * v;
		
	//Get T,B,N vectors
	T = normalize( nmat * t );
	B = -normalize( nmat * b );
	N = normalize( nmat * n );

	//Get tex coord	
	texcoord0 = gl_MultiTexCoord0.xy * vec2(0.25,0.5) + texcoordoffset;
	
	//Output color
	fragcolor=vec4(1.0);
}
