Const CHARACTER_EYELID_TIME_MIN:Int = 1000
Const CHARACTER_EYELID_TIME_MAX:Int = 4000
Const CHARACTER_EYELID_TIME_FREQ:Int = 50

'##################################################################################################
Type TCharacter
	
	'################################################
	'Main stuff.
	Field Path:String
	Field Mesh:TMesh
	Field Body:TBody
	Field AnimCount:Int
	Field AnimMapID:TMap
	Field AnimMapName:TMap
	
	'################################################
	'Eyelid Stuff.
	Field EyeLidEnabled:Int
	Field EyeLidMaterial:TMaterial
	Field EyeLidTex:TTexture[4]
	Field EyeLidSurface:TSurface
	Field EyeLidTimer:TTimer
	Field EyeLidState:Int
	
	Method DisableEyeLid()
		SetMaterialTexture( EyeLidMaterial, EyeLidTex[1], 0 )
		PaintSurface( EyeLidSurface, EyeLidMaterial )
		
		For Local X:Int = 0 To 3
			EyeLidTex[X] = Null
		 Next
		EyeLidMaterial = Null
		EyeLidSurface = Null
		EyeLidEnabled = False
	End Method
	
	Method EnableEyeLid( SurfaceID:Int )
		EyeLidMaterial = LoadMaterial( Path + "EyeLid.mat" )
		EyeLidTex[0] = LoadTexture( Path + "EyeLid_3.dds" )
		EyeLidTex[1] = LoadTexture( Path + "EyeLid_0.dds" )
		EyeLidTex[2] = LoadTexture( Path + "EyeLid_1.dds" )
		EyeLidTex[3] = LoadTexture( Path + "EyeLid_2.dds" )
		EyeLidSurface = GetSurface( Mesh, SurfaceID )
		CreateEyeLidTimer()
		EyeLidEnabled = True
	End Method
	
	Method CreateEyeLidTimer()
		EyeLidTimer = CreateTimer( 1000.0/Rand( CHARACTER_EYELID_TIME_MIN, CHARACTER_EYELID_TIME_MAX ) )
	End Method
	
	Method UpdateEyeLid()
		If TimerTicks( EyeLidTimer ) > 0
			StopTimer( EyeLidTimer )
			EyeLidTimer = Null
			
			Select EyeLidState
				Case 1, 2
					EyeLidTimer = CreateTimer( 1000.0/CHARACTER_EYELID_TIME_FREQ )
					EyeLidState = EyeLidState + 1
				Case 3
					EyeLidTimer = CreateTimer( 1000.0/CHARACTER_EYELID_TIME_FREQ )
					EyeLidState = 0
				Case 0
					CreateEyeLidTimer()
					EyeLidState:+1
			End Select
			
			SetMaterialTexture( EyeLidMaterial, EyeLidTex[ EyeLidState ], 0 )
			PaintSurface( EyeLidSurface, EyeLidMaterial )
		EndIf
	End Method
	
	'################################################
	Function Create:TCharacter( DirPath:String ) 
		Local A:TCharacter = New TCharacter
		A.Path = DirPath
		A.Mesh = LoadMesh( A.Path + "Mesh.gmf" )
		A.AnimMapID = CreateMap()
		A.AnimMapName = CreateMap()
		Return( A )
	End Function
	
	Method AddAnim( Name:String, StartFrame:Int, EndFrame:Int, Flags:Int = 0, Speed:Float = 0.5 )
		Local Anim:TAnimation = TAnimation.Create( Name, StartFrame, Endframe, Flags, Speed )
		MapInsert( AnimMapID, String( AnimCount ), Anim )
		MapInsert( AnimMapName, Name, Anim )	
		AnimCount:+1
	End Method
	
	Method AnimationFromID:TAnimation( ID:Int )
		Return TAnimation( MapValueForKey( AnimMapID, String( ID ) ) )
	End Method
	
	Method Animate( Anim:TAnimation )
		Anim.Play( Mesh )
	End Method
	
	'################################################
	Method Update()
		If EyeLidEnabled = True
			UpdateEyeLid()
		EndIf
	End Method
End Type

'##################################################################################################